@@ -41,7 +41,7 @@
 #include <stdio.h>
 
 #include "rcsid.h"
-RCSID("$Id: utmp.c,v 1.4 1997/05/01 23:15:00 marekm Exp $")
+RCSID("$Id: utmp.c,v 1.2 2001/02/11 10:50:13 tom Exp $")
 
 #if HAVE_UTMPX_H
 extern	struct	utmpx	utxent;
@@ -337,7 +337,7 @@ setutmp (name, line)
 
 	strncpy (utxline.ut_host, host, sizeof utxline.ut_host);
 
-#ifndef __hpux
+#if !defined(__hpux) && !defined(_AIX)
 	utxline.ut_syslen = strlen (utxline.ut_host) + 1;
 	if (utxline.ut_syslen > sizeof (utxline.ut_host))
 	  utxline.ut_syslen = sizeof (utxline.ut_host);
@@ -346,6 +346,7 @@ setutmp (name, line)
 	pututline (&utline);
 	pututxline (&utxline);
 
+#ifdef _WTMP_FILE
 	if ((fd = open (_WTMP_FILE "x", O_WRONLY|O_APPEND)) != -1) {
 		write (fd, (void *) &utxline, sizeof utxline);
 		close (fd);
@@ -354,6 +355,12 @@ setutmp (name, line)
 		write (fd, (void *) &utline, sizeof utline);
 		close (fd);
 	}
+#elif defined(_WTMPX_FILE)
+	if ((fd = open (_WTMPX_FILE, O_WRONLY|O_APPEND)) != -1) {
+		write (fd, (void *) &utxline, sizeof utxline);
+		close (fd);
+	}
+#endif /* !_WTMP_FILE */
 
 	utxent = utxline;
 	utent = utline;
@@ -362,6 +369,7 @@ setutmp (name, line)
 	struct	utmp	utmp;
 	int	fd;
 	int	found = 0;
+	off_t	pos;
 
 	if ((fd = open(_UTMP_FILE, O_RDWR)) < 0)
 		return;
@@ -417,8 +425,10 @@ setutmp (name, line)
 #ifdef HAVE_TTYSLOT
 	(void) lseek (fd, (off_t) (sizeof utmp) * ttyslot (), SEEK_SET);
 #else
-	if (found)	/* Back up a splot */
-		lseek (fd, (off_t) - sizeof utmp, SEEK_CUR);
+	if (found) {	/* Back up a slot */
+		pos = lseek (fd, (off_t) 0, SEEK_CUR);
+		lseek (fd, pos - sizeof utmp, SEEK_SET);
+	}
 	else		/* Otherwise, go to the end of the file */
 		lseek (fd, (off_t) 0, SEEK_END);
 #endif
