@@ -1,77 +1,77 @@
 
-/*  A Bison parser, made from ftpcmd.y with Bison version GNU Bison version 1.24
-  */
+/*  A Bison parser, made from ftpcmd.y
+    by GNU Bison version 1.28  */
 
 #define YYBISON 1  /* Identify Bison output.  */
 
-#define	A	258
-#define	B	259
-#define	C	260
-#define	E	261
-#define	F	262
-#define	I	263
-#define	L	264
-#define	N	265
-#define	P	266
-#define	R	267
-#define	S	268
-#define	T	269
-#define	SP	270
-#define	CRLF	271
-#define	COMMA	272
-#define	STRING	273
-#define	NUMBER	274
-#define	USER	275
-#define	PASS	276
-#define	ACCT	277
-#define	REIN	278
-#define	QUIT	279
-#define	PORT	280
-#define	PASV	281
-#define	TYPE	282
-#define	STRU	283
-#define	MODE	284
-#define	RETR	285
-#define	STOR	286
-#define	APPE	287
-#define	MLFL	288
-#define	MAIL	289
-#define	MSND	290
-#define	MSOM	291
-#define	MSAM	292
-#define	MRSQ	293
-#define	MRCP	294
-#define	ALLO	295
-#define	REST	296
-#define	RNFR	297
-#define	RNTO	298
-#define	ABOR	299
-#define	DELE	300
-#define	CWD	301
-#define	LIST	302
-#define	NLST	303
-#define	SITE	304
-#define	STAT	305
-#define	HELP	306
-#define	NOOP	307
-#define	MKD	308
-#define	RMD	309
-#define	PWD	310
-#define	CDUP	311
-#define	STOU	312
-#define	SMNT	313
-#define	SYST	314
-#define	SIZE	315
-#define	MDTM	316
-#define	AUTH	317
-#define	ADAT	318
-#define	PROT	319
-#define	PBSZ	320
-#define	CCC	321
-#define	UMASK	322
-#define	IDLE	323
-#define	CHMOD	324
-#define	LEXERR	325
+#define	A	257
+#define	B	258
+#define	C	259
+#define	E	260
+#define	F	261
+#define	I	262
+#define	L	263
+#define	N	264
+#define	P	265
+#define	R	266
+#define	S	267
+#define	T	268
+#define	SP	269
+#define	CRLF	270
+#define	COMMA	271
+#define	STRING	272
+#define	NUMBER	273
+#define	USER	274
+#define	PASS	275
+#define	ACCT	276
+#define	REIN	277
+#define	QUIT	278
+#define	PORT	279
+#define	PASV	280
+#define	TYPE	281
+#define	STRU	282
+#define	MODE	283
+#define	RETR	284
+#define	STOR	285
+#define	APPE	286
+#define	MLFL	287
+#define	MAIL	288
+#define	MSND	289
+#define	MSOM	290
+#define	MSAM	291
+#define	MRSQ	292
+#define	MRCP	293
+#define	ALLO	294
+#define	REST	295
+#define	RNFR	296
+#define	RNTO	297
+#define	ABOR	298
+#define	DELE	299
+#define	CWD	300
+#define	LIST	301
+#define	NLST	302
+#define	SITE	303
+#define	STAT	304
+#define	HELP	305
+#define	NOOP	306
+#define	MKD	307
+#define	RMD	308
+#define	PWD	309
+#define	CDUP	310
+#define	STOU	311
+#define	SMNT	312
+#define	SYST	313
+#define	SIZE	314
+#define	MDTM	315
+#define	AUTH	316
+#define	ADAT	317
+#define	PROT	318
+#define	PBSZ	319
+#define	CCC	320
+#define	UMASK	321
+#define	IDLE	322
+#define	CHMOD	323
+#define	LEXERR	324
 
 #line 42 "ftpcmd.y"
 
@@ -80,6 +80,9 @@
 static char sccsid[] = "@(#)ftpcmd.y	5.24 (Berkeley) 2/25/91";
 #endif /* not lint */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #include <sys/types.h>
 #include <sys/param.h>
@@ -115,6 +118,7 @@ extern	struct sockaddr_in his_addr, ctrl_addr;
 extern AUTH_DAT kdata;
 extern Key_schedule schedule;
 extern MSG_DAT msg_data;
+#define krb_get_err_text(R) krb_err_txt[R]
 #endif /* KERBEROS */
 #ifdef GSSAPI
 #include <gssapi/gssapi.h>
@@ -132,15 +136,8 @@ extern gss_ctx_id_t gcontext;
 #endif
 
 #ifndef NBBY
-#ifdef linux
 #define NBBY 8
 #endif
-#ifdef __pyrsoft
-#ifdef MIPSEB
-#define NBBY 8
-#endif
-#endif
-#endif
 
 extern	struct sockaddr_in data_dest;
 extern	int logged_in;
@@ -192,25 +189,8 @@ struct tab {
 struct tab cmdtab[];
 struct tab sitetab[];
 
-#line 161 "ftpcmd.y"
+#line 164 "ftpcmd.y"
 typedef union { int num; char *str; } YYSTYPE;
-
-#ifndef YYLTYPE
-typedef
-  struct yyltype
-    {
-      int timestamp;
-      int first_line;
-      int first_column;
-      int last_line;
-      int last_column;
-      char *text;
-   }
-  yyltype;
-
-#define YYLTYPE yyltype
-#endif
-
 #include <stdio.h>
 
 #ifndef __cplusplus
@@ -225,7 +205,7 @@ typedef
 #define	YYFLAG		-32768
 #define	YYNTBASE	71
 
-#define YYTRANSLATE(x) ((unsigned)(x) <= 325 ? yytranslate[x] : 87)
+#define YYTRANSLATE(x) ((unsigned)(x) <= 324 ? yytranslate[x] : 87)
 
 static const char yytranslate[] = {     0,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
@@ -253,14 +233,14 @@ static const char yytranslate[] = {     0,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
      2,     2,     2,     2,     2,     2,     2,     2,     2,     2,
-     2,     2,     2,     2,     2,     1,     2,     3,     4,     5,
-     6,     7,     8,     9,    10,    11,    12,    13,    14,    15,
-    16,    17,    18,    19,    20,    21,    22,    23,    24,    25,
-    26,    27,    28,    29,    30,    31,    32,    33,    34,    35,
-    36,    37,    38,    39,    40,    41,    42,    43,    44,    45,
-    46,    47,    48,    49,    50,    51,    52,    53,    54,    55,
-    56,    57,    58,    59,    60,    61,    62,    63,    64,    65,
-    66,    67,    68,    69,    70
+     2,     2,     2,     2,     2,     1,     3,     4,     5,     6,
+     7,     8,     9,    10,    11,    12,    13,    14,    15,    16,
+    17,    18,    19,    20,    21,    22,    23,    24,    25,    26,
+    27,    28,    29,    30,    31,    32,    33,    34,    35,    36,
+    37,    38,    39,    40,    41,    42,    43,    44,    45,    46,
+    47,    48,    49,    50,    51,    52,    53,    54,    55,    56,
+    57,    58,    59,    60,    61,    62,    63,    64,    65,    66,
+    67,    68,    69,    70
 };
 
 #if YYDEBUG != 0
@@ -317,16 +297,20 @@ static const short yyrhs[] = {    -1,
 
 #if YYDEBUG != 0
 static const short yyrline[] = { 0,
-   194,   195,   200,   203,   208,   213,   222,   227,   234,   238,
-   264,   299,   311,   323,   327,   331,   338,   345,   352,   357,
-   364,   369,   376,   383,   387,   394,   405,   409,   414,   421,
-   425,   440,   444,   451,   458,   463,   468,   472,   476,   486,
-   501,   515,   521,   535,   542,   566,   583,   605,   609,   614,
-   619,   624,   636,   645,   648,   652,   655,   658,   671,   675,
-   679,   685,   689,   693,   697,   703,   708,   713,   718,   723,
-   727,   732,   738,   745,   749,   753,   759,   763,   767,   773,
-   792,   795,   820
+   197,   198,   203,   206,   211,   216,   225,   230,   237,   241,
+   267,   302,   314,   326,   330,   334,   341,   348,   355,   360,
+   367,   372,   379,   386,   390,   397,   408,   412,   417,   424,
+   428,   443,   447,   454,   461,   466,   471,   475,   479,   489,
+   504,   518,   524,   538,   545,   569,   586,   608,   612,   617,
+   622,   627,   639,   648,   651,   655,   658,   661,   674,   678,
+   682,   688,   692,   696,   700,   706,   711,   716,   721,   726,
+   730,   735,   741,   748,   752,   756,   762,   766,   770,   776,
+   795,   798,   823
 };
+#endif
+
+
+#if YYDEBUG != 0 || defined (YYERROR_VERBOSE)
 
 static const char * const yytname[] = {   "$","error","$undefined.","A","B",
 "C","E","F","I","L","N","P","R","S","T","SP","CRLF","COMMA","STRING","NUMBER",
@@ -336,7 +320,7 @@ static const char * const yytname[] = {   "$","error","$undefined.","A","B",
 "MKD","RMD","PWD","CDUP","STOU","SMNT","SYST","SIZE","MDTM","AUTH","ADAT","PROT",
 "PBSZ","CCC","UMASK","IDLE","CHMOD","LEXERR","cmd_list","cmd","rcmd","username",
 "password","byte_size","host_port","form_code","prot_code","type_code","struct_code",
-"mode_code","pathname","pathstring","octal_number","check_login",""
+"mode_code","pathname","pathstring","octal_number","check_login", NULL
 };
 #endif
 
@@ -480,7 +464,8 @@ static const short yycheck[] = {   102,
     19,    17,    -1,    17,    19,    -1,    19
 };
 /* -*-C-*-  Note some compilers choke on comments on `#line' lines.  */
-#line 3 "/usr/local/share/bison.simple"
+#line 3 "/usr/lib/bison.simple"
+/* This file comes from bison-1.28.  */
 
 /* Skeleton output parser for bison,
    Copyright (C) 1984, 1989, 1990 Free Software Foundation, Inc.
@@ -497,47 +482,67 @@ static const short yycheck[] = {   102,
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+   Foundation, Inc., 59 Temple Place - Suite 330,
+   Boston, MA 02111-1307, USA.  */
 
 /* As a special exception, when this file is copied by Bison into a
    Bison output file, you may use that output file without restriction.
    This special exception was added by the Free Software Foundation
    in version 1.24 of Bison.  */
 
-#ifndef alloca
-#ifdef __GNUC__
-#define alloca __builtin_alloca
-#else /* not GNU C.  */
-#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || defined(__hpux)
-#include <alloca.h>
-#else /* not sparc */
-#if defined (MSDOS) && !defined (__TURBOC__)
-#include <malloc.h>
-#else /* not MSDOS, or __TURBOC__ */
-#if defined(_AIX)
-#include <malloc.h>
- #pragma alloca
-#else /* not MSDOS, __TURBOC__, or _AIX */
-#ifdef __hpux
-#ifdef __cplusplus
-extern "C" {
-void *alloca (unsigned int);
-};
-#else /* not __cplusplus */
-void *alloca ();
-#endif /* not __cplusplus */
-#endif /* __hpux */
-#endif /* not _AIX */
-#endif /* not MSDOS, or __TURBOC__ */
-#endif /* not sparc.  */
-#endif /* not GNU C.  */
-#endif /* alloca not defined.  */
-
 /* This is the parser code that is written into each bison parser
   when the %semantic_parser declaration is not specified in the grammar.
   It was written by Richard Stallman by simplifying the hairy parser
   used when %semantic_parser is specified.  */
 
+#ifndef YYSTACK_USE_ALLOCA
+#ifdef alloca
+#define YYSTACK_USE_ALLOCA
+#else /* alloca not defined */
+#ifdef __GNUC__
+#define YYSTACK_USE_ALLOCA
+#define alloca __builtin_alloca
+#else /* not GNU C.  */
+#if (!defined (__STDC__) && defined (sparc)) || defined (__sparc__) || defined (__sparc) || defined (__sgi) || (defined (__sun) && defined (__i386))
+#define YYSTACK_USE_ALLOCA
+#include <alloca.h>
+#else /* not sparc */
+/* We think this test detects Watcom and Microsoft C.  */
+/* This used to test MSDOS, but that is a bad idea
+   since that symbol is in the user namespace.  */
+#if (defined (_MSDOS) || defined (_MSDOS_)) && !defined (__TURBOC__)
+#if 0 /* No need for malloc.h, which pollutes the namespace;
+	 instead, just don't use alloca.  */
+#include <malloc.h>
+#endif
+#else /* not MSDOS, or __TURBOC__ */
+#if defined(_AIX)
+/* I don't know what this was needed for, but it pollutes the namespace.
+   So I turned it off.   rms, 2 May 1997.  */
+/* #include <malloc.h>  */
+ #pragma alloca
+#define YYSTACK_USE_ALLOCA
+#else /* not MSDOS, or __TURBOC__, or _AIX */
+#if 0
+#ifdef __hpux /* haible@ilog.fr says this works for HPUX 9.05 and up,
+		 and on HPUX 10.  Eventually we can turn this on.  */
+#define YYSTACK_USE_ALLOCA
+#define alloca __builtin_alloca
+#endif /* __hpux */
+#endif
+#endif /* not _AIX */
+#endif /* not MSDOS, or __TURBOC__ */
+#endif /* not sparc */
+#endif /* not GNU C */
+#endif /* alloca not defined */
+#endif /* YYSTACK_USE_ALLOCA not defined */
+
+#ifdef YYSTACK_USE_ALLOCA
+#define YYSTACK_ALLOC alloca
+#else
+#define YYSTACK_ALLOC malloc
+#endif
+
 /* Note: there must be only one dollar sign in this file.
    It is replaced by the list of actions, each action
    as one case of the switch.  */
@@ -546,8 +551,8 @@ void *alloca ();
 #define yyclearin	(yychar = YYEMPTY)
 #define YYEMPTY		-2
 #define YYEOF		0
-#define YYACCEPT	return(0)
-#define YYABORT 	return(1)
+#define YYACCEPT	goto yyacceptlab
+#define YYABORT 	goto yyabortlab
 #define YYERROR		goto yyerrlab1
 /* Like YYERROR except do call yyerror.
    This remains here temporarily to ease the
@@ -628,24 +633,24 @@ int yydebug;			/*  nonzero means print parse trace	*/
 #ifndef YYMAXDEPTH
 #define YYMAXDEPTH 10000
 #endif
-
-/* Prevent warning if -Wstrict-prototypes.  */
-#ifdef __GNUC__
-int yyparse (void);
-#endif
 
+/* Define __yy_memcpy.  Note that the size argument
+   should be passed with type unsigned int, because that is what the non-GCC
+   definitions require.  With GCC, __builtin_memcpy takes an arg
+   of type size_t, but it can handle unsigned int.  */
+
 #if __GNUC__ > 1		/* GNU C and GNU C++ define this.  */
-#define __yy_memcpy(FROM,TO,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
+#define __yy_memcpy(TO,FROM,COUNT)	__builtin_memcpy(TO,FROM,COUNT)
 #else				/* not GNU C or C++ */
 #ifndef __cplusplus
 
 /* This is the most reliable way to avoid incompatibilities
    in available built-in functions on various systems.  */
 static void
-__yy_memcpy (from, to, count)
-     char *from;
+__yy_memcpy (to, from, count)
      char *to;
-     int count;
+     char *from;
+     unsigned int count;
 {
   register char *f = from;
   register char *t = to;
@@ -660,10 +665,10 @@ __yy_memcpy (from, to, count)
 /* This is the most reliable way to avoid incompatibilities
    in available built-in functions on various systems.  */
 static void
-__yy_memcpy (char *from, char *to, int count)
+__yy_memcpy (char *to, char *from, unsigned int count)
 {
-  register char *f = from;
   register char *t = to;
+  register char *f = from;
   register int i = count;
 
   while (i-- > 0)
@@ -673,7 +678,7 @@ __yy_memcpy (char *from, char *to, int count)
 #endif
 #endif
 
-#line 192 "/usr/local/share/bison.simple"
+#line 217 "/usr/lib/bison.simple"
 
 /* The user can define YYPARSE_PARAM as the name of an argument to be passed
    into yyparse.  The argument should have type void *.
@@ -682,14 +687,29 @@ __yy_memcpy (char *from, char *to, int count)
    to the proper pointer type.  */
 
 #ifdef YYPARSE_PARAM
-#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
-#else
-#define YYPARSE_PARAM
+#ifdef __cplusplus
+#define YYPARSE_PARAM_ARG void *YYPARSE_PARAM
 #define YYPARSE_PARAM_DECL
+#else /* not __cplusplus */
+#define YYPARSE_PARAM_ARG YYPARSE_PARAM
+#define YYPARSE_PARAM_DECL void *YYPARSE_PARAM;
+#endif /* not __cplusplus */
+#else /* not YYPARSE_PARAM */
+#define YYPARSE_PARAM_ARG
+#define YYPARSE_PARAM_DECL
+#endif /* not YYPARSE_PARAM */
+
+/* Prevent warning if -Wstrict-prototypes.  */
+#ifdef __GNUC__
+#ifdef YYPARSE_PARAM
+int yyparse (void *);
+#else
+int yyparse (void);
+#endif
 #endif
 
 int
-yyparse(YYPARSE_PARAM)
+yyparse(YYPARSE_PARAM_ARG)
      YYPARSE_PARAM_DECL
 {
   register int yystate;
@@ -716,6 +736,7 @@ yyparse(YYPARSE_PARAM)
 #endif
 
   int yystacksize = YYINITDEPTH;
+  int yyfree_stacks = 0;
 
 #ifdef YYPURE
   int yychar;
@@ -800,18 +821,32 @@ yynewstate:
       if (yystacksize >= YYMAXDEPTH)
 	{
 	  yyerror("parser stack overflow");
+	  if (yyfree_stacks)
+	    {
+	      free (yyss);
+	      free (yyvs);
+#ifdef YYLSP_NEEDED
+	      free (yyls);
+#endif
+	    }
 	  return 2;
 	}
       yystacksize *= 2;
       if (yystacksize > YYMAXDEPTH)
 	yystacksize = YYMAXDEPTH;
-      yyss = (short *) alloca (yystacksize * sizeof (*yyssp));
-      __yy_memcpy ((char *)yyss1, (char *)yyss, size * sizeof (*yyssp));
-      yyvs = (YYSTYPE *) alloca (yystacksize * sizeof (*yyvsp));
-      __yy_memcpy ((char *)yyvs1, (char *)yyvs, size * sizeof (*yyvsp));
+#ifndef YYSTACK_USE_ALLOCA
+      yyfree_stacks = 1;
+#endif
+      yyss = (short *) YYSTACK_ALLOC (yystacksize * sizeof (*yyssp));
+      __yy_memcpy ((char *)yyss, (char *)yyss1,
+		   size * (unsigned int) sizeof (*yyssp));
+      yyvs = (YYSTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yyvsp));
+      __yy_memcpy ((char *)yyvs, (char *)yyvs1,
+		   size * (unsigned int) sizeof (*yyvsp));
 #ifdef YYLSP_NEEDED
-      yyls = (YYLTYPE *) alloca (yystacksize * sizeof (*yylsp));
-      __yy_memcpy ((char *)yyls1, (char *)yyls, size * sizeof (*yylsp));
+      yyls = (YYLTYPE *) YYSTACK_ALLOC (yystacksize * sizeof (*yylsp));
+      __yy_memcpy ((char *)yyls, (char *)yyls1,
+		   size * (unsigned int) sizeof (*yylsp));
 #endif
 #endif /* no yyoverflow */
 
@@ -972,28 +1007,28 @@ yyreduce:
   switch (yyn) {
 
 case 2:
-#line 196 "ftpcmd.y"
+#line 199 "ftpcmd.y"
 {
 			fromname = (char *) 0;
 			restart_point = (off_t) 0;
 		;
     break;}
 case 4:
-#line 204 "ftpcmd.y"
+#line 207 "ftpcmd.y"
 {
 			user((char *) yyvsp[-1].str);
 			free((char *) yyvsp[-1].str);
 		;
     break;}
 case 5:
-#line 209 "ftpcmd.y"
+#line 212 "ftpcmd.y"
 {
 			pass((char *) yyvsp[-1].str);
 			free((char *) yyvsp[-1].str);
 		;
     break;}
 case 6:
-#line 214 "ftpcmd.y"
+#line 217 "ftpcmd.y"
 {
 			usedefault = 0;
 			if (pdata >= 0) {
@@ -1004,14 +1039,14 @@ case 6:
 		;
     break;}
 case 7:
-#line 223 "ftpcmd.y"
+#line 226 "ftpcmd.y"
 {
 			if (yyvsp[-1].num)
 				passive();
 		;
     break;}
 case 8:
-#line 228 "ftpcmd.y"
+#line 231 "ftpcmd.y"
 {
 		    if (maxbuf)
 			setlevel (yyvsp[-1].num);
@@ -1020,13 +1055,13 @@ case 8:
 		;
     break;}
 case 9:
-#line 235 "ftpcmd.y"
+#line 238 "ftpcmd.y"
 {
 			reply(534, "CCC not supported");
 		;
     break;}
 case 10:
-#line 239 "ftpcmd.y"
+#line 242 "ftpcmd.y"
 {
 			/* Others may want to do something more fancy here */
 			if (!auth_type)
@@ -1054,7 +1089,7 @@ case 10:
 		;
     break;}
 case 11:
-#line 265 "ftpcmd.y"
+#line 268 "ftpcmd.y"
 {
 			switch (cmd_type) {
 
@@ -1091,7 +1126,7 @@ case 11:
 		;
     break;}
 case 12:
-#line 300 "ftpcmd.y"
+#line 303 "ftpcmd.y"
 {
 			switch (yyvsp[-1].num) {
 
@@ -1105,7 +1140,7 @@ case 12:
 		;
     break;}
 case 13:
-#line 312 "ftpcmd.y"
+#line 315 "ftpcmd.y"
 {
 			switch (yyvsp[-1].num) {
 
@@ -1119,19 +1154,19 @@ case 13:
 		;
     break;}
 case 14:
-#line 324 "ftpcmd.y"
+#line 327 "ftpcmd.y"
 {
 			reply(202, "ALLO command ignored.");
 		;
     break;}
 case 15:
-#line 328 "ftpcmd.y"
+#line 331 "ftpcmd.y"
 {
 			reply(202, "ALLO command ignored.");
 		;
     break;}
 case 16:
-#line 332 "ftpcmd.y"
+#line 335 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				retrieve((char *) 0, (char *) yyvsp[-1].str);
@@ -1140,7 +1175,7 @@ case 16:
 		;
     break;}
 case 17:
-#line 339 "ftpcmd.y"
+#line 342 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				store_file((char *) yyvsp[-1].str, "w", 0);
@@ -1149,7 +1184,7 @@ case 17:
 		;
     break;}
 case 18:
-#line 346 "ftpcmd.y"
+#line 349 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				store_file((char *) yyvsp[-1].str, "a", 0);
@@ -1158,14 +1193,14 @@ case 18:
 		;
     break;}
 case 19:
-#line 353 "ftpcmd.y"
+#line 356 "ftpcmd.y"
 {
 			if (yyvsp[-1].num)
 				send_file_list(".");
 		;
     break;}
 case 20:
-#line 358 "ftpcmd.y"
+#line 361 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL) 
 				send_file_list((char *) yyvsp[-1].str);
@@ -1174,14 +1209,14 @@ case 20:
 		;
     break;}
 case 21:
-#line 365 "ftpcmd.y"
+#line 368 "ftpcmd.y"
 {
 			if (yyvsp[-1].num)
 				retrieve("/bin/ls -lgA", "");
 		;
     break;}
 case 22:
-#line 370 "ftpcmd.y"
+#line 373 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				retrieve("/bin/ls -lgA %s", (char *) yyvsp[-1].str);
@@ -1190,7 +1225,7 @@ case 22:
 		;
     break;}
 case 23:
-#line 377 "ftpcmd.y"
+#line 380 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				statfilecmd((char *) yyvsp[-1].str);
@@ -1199,13 +1234,13 @@ case 23:
 		;
     break;}
 case 24:
-#line 384 "ftpcmd.y"
+#line 387 "ftpcmd.y"
 {
 			statcmd();
 		;
     break;}
 case 25:
-#line 388 "ftpcmd.y"
+#line 391 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				delete_file((char *) yyvsp[-1].str);
@@ -1214,7 +1249,7 @@ case 25:
 		;
     break;}
 case 26:
-#line 395 "ftpcmd.y"
+#line 398 "ftpcmd.y"
 {
 			if (fromname) {
 				renamecmd(fromname, (char *) yyvsp[-1].str);
@@ -1227,20 +1262,20 @@ case 26:
 		;
     break;}
 case 27:
-#line 406 "ftpcmd.y"
+#line 409 "ftpcmd.y"
 {
 			reply(225, "ABOR command successful.");
 		;
     break;}
 case 28:
-#line 410 "ftpcmd.y"
+#line 413 "ftpcmd.y"
 {
 			if (yyvsp[-1].num)
 				cwd(pw->pw_dir);
 		;
     break;}
 case 29:
-#line 415 "ftpcmd.y"
+#line 418 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				cwd((char *) yyvsp[-1].str);
@@ -1249,13 +1284,13 @@ case 29:
 		;
     break;}
 case 30:
-#line 422 "ftpcmd.y"
+#line 425 "ftpcmd.y"
 {
 			help(cmdtab, (char *) 0);
 		;
     break;}
 case 31:
-#line 426 "ftpcmd.y"
+#line 429 "ftpcmd.y"
 {
 			register char *cp = (char *)yyvsp[-1].str;
 
@@ -1272,13 +1307,13 @@ case 31:
 		;
     break;}
 case 32:
-#line 441 "ftpcmd.y"
+#line 444 "ftpcmd.y"
 {
 			reply(200, "NOOP command successful.");
 		;
     break;}
 case 33:
-#line 445 "ftpcmd.y"
+#line 448 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				makedir((char *) yyvsp[-1].str);
@@ -1287,7 +1322,7 @@ case 33:
 		;
     break;}
 case 34:
-#line 452 "ftpcmd.y"
+#line 455 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				removedir((char *) yyvsp[-1].str);
@@ -1296,33 +1331,33 @@ case 34:
 		;
     break;}
 case 35:
-#line 459 "ftpcmd.y"
+#line 462 "ftpcmd.y"
 {
 			if (yyvsp[-1].num)
 				pwd();
 		;
     break;}
 case 36:
-#line 464 "ftpcmd.y"
+#line 467 "ftpcmd.y"
 {
 			if (yyvsp[-1].num)
 				cwd("..");
 		;
     break;}
 case 37:
-#line 469 "ftpcmd.y"
+#line 472 "ftpcmd.y"
 {
 			help(sitetab, (char *) 0);
 		;
     break;}
 case 38:
-#line 473 "ftpcmd.y"
+#line 476 "ftpcmd.y"
 {
 			help(sitetab, (char *) yyvsp[-1].str);
 		;
     break;}
 case 39:
-#line 477 "ftpcmd.y"
+#line 480 "ftpcmd.y"
 {
 			int oldmask;
 
@@ -1334,7 +1369,7 @@ case 39:
 		;
     break;}
 case 40:
-#line 487 "ftpcmd.y"
+#line 490 "ftpcmd.y"
 {
 			int oldmask;
 
@@ -1351,7 +1386,7 @@ case 40:
 		;
     break;}
 case 41:
-#line 502 "ftpcmd.y"
+#line 505 "ftpcmd.y"
 {
 			if (yyvsp[-5].num && (yyvsp[-1].str != NULL)) {
 				if (yyvsp[-3].num > 0777)
@@ -1367,7 +1402,7 @@ case 41:
 		;
     break;}
 case 42:
-#line 516 "ftpcmd.y"
+#line 519 "ftpcmd.y"
 {
 			reply(200,
 			    "Current IDLE time limit is %d seconds; max %d",
@@ -1375,7 +1410,7 @@ case 42:
 		;
     break;}
 case 43:
-#line 522 "ftpcmd.y"
+#line 525 "ftpcmd.y"
 {
 			if (yyvsp[-1].num < 30 || yyvsp[-1].num > maxtimeout) {
 				reply(501,
@@ -1391,7 +1426,7 @@ case 43:
 		;
     break;}
 case 44:
-#line 536 "ftpcmd.y"
+#line 539 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				store_file((char *) yyvsp[-1].str, "w", 1);
@@ -1400,7 +1435,7 @@ case 44:
 		;
     break;}
 case 45:
-#line 543 "ftpcmd.y"
+#line 546 "ftpcmd.y"
 {
 #ifdef unix
 #ifdef __svr4__
@@ -1418,7 +1453,7 @@ case 45:
 		;
     break;}
 case 46:
-#line 567 "ftpcmd.y"
+#line 570 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL)
 				sizecmd((char *) yyvsp[-1].str);
@@ -1427,7 +1462,7 @@ case 46:
 		;
     break;}
 case 47:
-#line 584 "ftpcmd.y"
+#line 587 "ftpcmd.y"
 {
 			if (yyvsp[-3].num && yyvsp[-1].str != NULL) {
 				struct stat stbuf;
@@ -1451,33 +1486,33 @@ case 47:
 		;
     break;}
 case 48:
-#line 606 "ftpcmd.y"
+#line 609 "ftpcmd.y"
 {
 			auth((char *) yyvsp[-1].str);
 		;
     break;}
 case 49:
-#line 610 "ftpcmd.y"
+#line 613 "ftpcmd.y"
 {
 			auth_data((char *) yyvsp[-1].str);
 			free((char *) yyvsp[-1].str);
 		;
     break;}
 case 50:
-#line 615 "ftpcmd.y"
+#line 618 "ftpcmd.y"
 {
 			reply(221, "Goodbye.");
 			dologout(0);
 		;
     break;}
 case 51:
-#line 620 "ftpcmd.y"
+#line 623 "ftpcmd.y"
 {
 			yyerrok;
 		;
     break;}
 case 52:
-#line 625 "ftpcmd.y"
+#line 628 "ftpcmd.y"
 {
 			char *renamefrom();
 
@@ -1491,7 +1526,7 @@ case 52:
 		;
     break;}
 case 53:
-#line 637 "ftpcmd.y"
+#line 640 "ftpcmd.y"
 {
 			fromname = (char *) 0;
 			restart_point = yyvsp[-1].num;
@@ -1500,13 +1535,13 @@ case 53:
 		;
     break;}
 case 55:
-#line 649 "ftpcmd.y"
+#line 652 "ftpcmd.y"
 {
 			*(char **)&(yyval.str) = (char *)calloc(1, sizeof(char));
 		;
     break;}
 case 58:
-#line 660 "ftpcmd.y"
+#line 663 "ftpcmd.y"
 {
 			register char *a, *p;
 
@@ -1518,140 +1553,140 @@ case 58:
 		;
     break;}
 case 59:
-#line 672 "ftpcmd.y"
+#line 675 "ftpcmd.y"
 {
 		yyval.num = FORM_N;
 	;
     break;}
 case 60:
-#line 676 "ftpcmd.y"
+#line 679 "ftpcmd.y"
 {
 		yyval.num = FORM_T;
 	;
     break;}
 case 61:
-#line 680 "ftpcmd.y"
+#line 683 "ftpcmd.y"
 {
 		yyval.num = FORM_C;
 	;
     break;}
 case 62:
-#line 686 "ftpcmd.y"
+#line 689 "ftpcmd.y"
 {
 		yyval.num = PROT_C;
 	;
     break;}
 case 63:
-#line 690 "ftpcmd.y"
+#line 693 "ftpcmd.y"
 {
 		yyval.num = PROT_S;
 	;
     break;}
 case 64:
-#line 694 "ftpcmd.y"
+#line 697 "ftpcmd.y"
 {
 		yyval.num = PROT_P;
 	;
     break;}
 case 65:
-#line 698 "ftpcmd.y"
+#line 701 "ftpcmd.y"
 {
 		yyval.num = PROT_E;
 	;
     break;}
 case 66:
-#line 704 "ftpcmd.y"
+#line 707 "ftpcmd.y"
 {
 		cmd_type = TYPE_A;
 		cmd_form = FORM_N;
 	;
     break;}
 case 67:
-#line 709 "ftpcmd.y"
+#line 712 "ftpcmd.y"
 {
 		cmd_type = TYPE_A;
 		cmd_form = yyvsp[0].num;
 	;
     break;}
 case 68:
-#line 714 "ftpcmd.y"
+#line 717 "ftpcmd.y"
 {
 		cmd_type = TYPE_E;
 		cmd_form = FORM_N;
 	;
     break;}
 case 69:
-#line 719 "ftpcmd.y"
+#line 722 "ftpcmd.y"
 {
 		cmd_type = TYPE_E;
 		cmd_form = yyvsp[0].num;
 	;
     break;}
 case 70:
-#line 724 "ftpcmd.y"
+#line 727 "ftpcmd.y"
 {
 		cmd_type = TYPE_I;
 	;
     break;}
 case 71:
-#line 728 "ftpcmd.y"
+#line 731 "ftpcmd.y"
 {
 		cmd_type = TYPE_L;
 		cmd_bytesz = NBBY;
 	;
     break;}
 case 72:
-#line 733 "ftpcmd.y"
+#line 736 "ftpcmd.y"
 {
 		cmd_type = TYPE_L;
 		cmd_bytesz = yyvsp[0].num;
 	;
     break;}
 case 73:
-#line 739 "ftpcmd.y"
+#line 742 "ftpcmd.y"
 {
 		cmd_type = TYPE_L;
 		cmd_bytesz = yyvsp[0].num;
 	;
     break;}
 case 74:
-#line 746 "ftpcmd.y"
+#line 749 "ftpcmd.y"
 {
 		yyval.num = STRU_F;
 	;
     break;}
 case 75:
-#line 750 "ftpcmd.y"
+#line 753 "ftpcmd.y"
 {
 		yyval.num = STRU_R;
 	;
     break;}
 case 76:
-#line 754 "ftpcmd.y"
+#line 757 "ftpcmd.y"
 {
 		yyval.num = STRU_P;
 	;
     break;}
 case 77:
-#line 760 "ftpcmd.y"
+#line 763 "ftpcmd.y"
 {
 		yyval.num = MODE_S;
 	;
     break;}
 case 78:
-#line 764 "ftpcmd.y"
+#line 767 "ftpcmd.y"
 {
 		yyval.num = MODE_B;
 	;
     break;}
 case 79:
-#line 768 "ftpcmd.y"
+#line 771 "ftpcmd.y"
 {
 		yyval.num = MODE_C;
 	;
     break;}
 case 80:
-#line 774 "ftpcmd.y"
+#line 777 "ftpcmd.y"
 {
 		/*
 		 * Problem: this production is used for all pathname
@@ -1670,7 +1705,7 @@ case 80:
 	;
     break;}
 case 82:
-#line 796 "ftpcmd.y"
+#line 799 "ftpcmd.y"
 {
 		register int ret, dec, multby, digit;
 
@@ -1695,7 +1730,7 @@ case 82:
 	;
     break;}
 case 83:
-#line 821 "ftpcmd.y"
+#line 824 "ftpcmd.y"
 {
 		if (logged_in)
 			yyval.num = 1;
@@ -1707,7 +1742,7 @@ case 83:
     break;}
 }
    /* the action file gets copied in in place of this dollarsign */
-#line 487 "/usr/local/share/bison.simple"
+#line 543 "/usr/lib/bison.simple"
 
   yyvsp -= yylen;
   yyssp -= yylen;
@@ -1902,8 +1937,32 @@ yyerrhandle:
 
   yystate = yyn;
   goto yynewstate;
+
+ yyacceptlab:
+  /* YYACCEPT comes here.  */
+  if (yyfree_stacks)
+    {
+      free (yyss);
+      free (yyvs);
+#ifdef YYLSP_NEEDED
+      free (yyls);
+#endif
+    }
+  return 0;
+
+ yyabortlab:
+  /* YYABORT comes here.  */
+  if (yyfree_stacks)
+    {
+      free (yyss);
+      free (yyvs);
+#ifdef YYLSP_NEEDED
+      free (yyls);
+#endif
+    }
+  return 1;
 }
-#line 831 "ftpcmd.y"
+#line 834 "ftpcmd.y"
 
 
 struct tab cmdtab[] = {		/* In order defined in RFC 765 */
