@@ -99,7 +99,7 @@ static unsigned char bigexp[] = {
   0x08, 0x19, 0x2A, 0x3B, 0x4C, 0x5D, 0x6E, 0x7F
 };
 
-#define NRUNS 10
+#define NRUNS 20
 #define PERF_NORM 671	/* Standardized to 167MHz UltraSparc-1 */
 
 double
@@ -137,14 +137,14 @@ bench_modexp(b, e, m)
 #endif
 }
 
+int
 main(argc, argv)
      int argc;
      char **argv;
 {
   BigInteger b, m, e, r;
   char hexbuf[MAXHEXPARAMLEN];
-  double t;
-  int tb, te, tg;
+  double tb, te, tg;
 
   printf("math library test: ");
   printf("0x%s ^ ", t_tohex(hexbuf, testbase, sizeof(testbase)));
@@ -169,18 +169,18 @@ main(argc, argv)
   b = BigIntegerFromBytes(smallbase, sizeof(smallbase));
   e = BigIntegerFromBytes(bigexp, sizeof(bigexp));
   tg = (int) (1000 * bench_modexp(b, e, m));
-  printf("t[g] = %d ms\n", tg);
+  printf("t[g] = %.1lf ms\n", tg);
   BigIntegerFree(b);
   b = BigIntegerFromBytes(bigbase, sizeof(bigbase));
-  tb = (int) (1000 * bench_modexp(b, e, m));
-  printf("t[b] = %d ms\n", tb);
+  tb = 1000.0 * bench_modexp(b, e, m);
+  printf("t[b] = %.1lf ms\n", tb);
   BigIntegerFree(e);
   e = BigIntegerFromBytes(smallexp, sizeof(smallexp));
-  te = (int) (1000 * bench_modexp(b, e, m));
-  printf("t[e] = %d ms\n", te);
+  te = 1000.0 * bench_modexp(b, e, m);
+  printf("t[e] = %.1lf ms\n", te);
 
-  printf("\nSRPbench total:  %d\n", tb + tg + te);
-  printf("Performance index:  %.3f\n", (double) PERF_NORM / (tb + tg + te));
+  printf("\nSRPbench total:  %.1lf\n", tb + tg + te);
+  printf("Performance index:  %.3lf\n", (double) PERF_NORM / (tb + tg + te));
 
   BigIntegerFree(b);
   BigIntegerFree(e);
