@@ -44,7 +44,11 @@
 #include "t_pwd.h"
 #include "t_sha.h"
 
+#ifdef NO_ERRNO
+int errno = -1;
+#else
 extern int errno;
+#endif
 
 main(argc, argv)
      int argc;
@@ -65,7 +69,24 @@ main(argc, argv)
   t_random(hexbuf, 64);
   printf("Strong random numbers:\n");
   for(i = 0; i < 64; ++i)
-    printf(" %02X", (unsigned char) hexbuf[i]);
+    printf("%02X ", (unsigned char) hexbuf[i]);
+
+  printf("\n\nSHA-1 Test Vector (expected output):\n");
+  printf("A9 99 3E 36 47 06 81 6A BA 3E 25 71 78 50 C2 6C 9C D0 D8 9D\n");
+  SHA1Init(&ctxt);
+  SHA1Update(&ctxt, "abc", 3);
+  SHA1Final(dout, &ctxt);
+  printf("SHA-1('abc'):\n");
+  for(i = 0; i < 20; ++i)
+    printf("%02X ", (unsigned char) dout[i]);
+  SHA1Init(&ctxt);
+  SHA1Update(&ctxt, "a", 1);
+  SHA1Update(&ctxt, "b", 1);
+  SHA1Update(&ctxt, "c", 1);
+  SHA1Final(dout, &ctxt);
+  printf("\nSHA-1('a'||'b'||'c'):\n");
+  for(i = 0; i < 20; ++i)
+    printf("%02X ", (unsigned char) dout[i]);
 
   printf("\n");
 
@@ -137,7 +158,7 @@ main(argc, argv)
     t_closepw(tpw);
   }
   else
-    while(printf("login: "), gets(username) != NULL)
+    while(printf("login: "), (gets(username) != NULL && *username != '\0'))
     {
       t_getpass(pass, 128, "Enter a password:");
       if(t_verifypw(username, pass) > 0)
