@@ -45,6 +45,8 @@
 #include "t_client.h"
 #include "t_sha.h"
 
+extern int t_isprime(BigInteger);
+
 _TYPE( struct t_client * )
 t_clientopen(u, n, g, s)
      const char * u;
@@ -55,7 +57,47 @@ t_clientopen(u, n, g, s)
   struct t_client * tc;
   unsigned char buf1[SHA_DIGESTSIZE], buf2[SHA_DIGESTSIZE];
   SHA1_CTX ctxt;
-  int i;
+  int i, validated;
+  struct t_preconf * tpc;
+
+  BigInteger nn, gg, n12, r;
+
+  validated = 0;
+  if(n->len < MIN_MOD_BYTES)
+    return 0;
+  for(i = 0; i < t_getprecount(); ++i) {
+    tpc = t_getpreparam(i);
+    if(tpc->modulus.len == n->len && tpc->generator.len == g->len &&
+       memcmp(tpc->modulus.data, n->data, n->len) == 0 &&
+       memcmp(tpc->generator.data, g->data, g->len) == 0) {
+      validated = 1;	/* Match found, done */
+      break;
+    }
+  }
+
+  if(validated == 0) {
+    /* Not in builtin list, must test */
+    nn = BigIntegerFromBytes(n->data, n->len);
+    gg = BigIntegerFromBytes(g->data, g->len);
+    /* n12 == (n-1)/2 */
+    n12 = BigIntegerFromInt(0);
+    BigIntegerSubInt(n12, nn, 1);
+    BigIntegerDivInt(n12, n12, 2);
+    if(t_isprime(n12) && t_isprime(nn)) {
+      r = BigIntegerFromInt(0);		/* Check g^n12 == -1 (mod n) */
+      BigIntegerModExp(r, gg, n12, nn);
+      BigIntegerAddInt(r, r, 1);
+      if(BigIntegerCmp(r, nn) == 0)
+	validated = 1;
+      BigIntegerFree(r);
+    }
+    BigIntegerFree(nn);
+    BigIntegerFree(gg);
+    BigIntegerFree(n12);
+  }
+
+  if(validated == 0)
+    return 0;
 
   if((tc = malloc(sizeof(struct t_client))) == 0)
     return 0;
