@@ -43,14 +43,16 @@
 #define T_DEFINES_H
 
 #ifndef P
-#if __STDC__
+#if defined(__STDC__) || defined(__cplusplus)
 #define P(x) x
 #else
 #define P(x) ()
 #endif
 #endif
 
+#ifdef HAVE_CONFIG_H
 #include "config.h"
+#endif /* HAVE_CONFIG_H */
 
 #ifndef _DLLDECL
 #define _DLLDECL
@@ -91,6 +93,8 @@ char *strchr(), *strrchr(), *strtok();
 #endif
 #endif /* not STDC_HEADERS */
 
+#include <sys/types.h>
+
 #if TIME_WITH_SYS_TIME
 #include <sys/time.h>
 #include <time.h>
@@ -135,6 +139,7 @@ char *strchr(), *strrchr(), *strtok();
 
 /* Should be defined by configure; GNU_MP is default */
 /* #define GNU_MP 1 */
+/* #define OPENSSL 1 */
 /* #define CRYPTOLIB 1 */
 
 #ifndef MATH_PRIV
