@@ -44,10 +44,13 @@
 
 #include "config.h"
 
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+# include "openssl/bn.h"
+typedef BIGNUM * BigInteger;
+#elif defined(CRYPTOLIB)
 # include "libcrypt.h"
 typedef BigInt BigInteger;
-#elif GNU_MP
+#elif defined(GNU_MP)
 # include "gmp.h"
 typedef MP_INT * BigInteger;
 #else
@@ -64,9 +67,13 @@ BigInteger
 BigIntegerFromInt(n)
      unsigned int n;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BIGNUM * a = BN_new();
+  BN_set_word(a, n);
+  return a;
+#elif defined(CRYPTOLIB)
   return bigInit(n);
-#elif GNU_MP
+#elif defined(GNU_MP)
   BigInteger rv;
 
   rv = (BigInteger) malloc(sizeof(MP_INT));
@@ -80,7 +87,11 @@ BigIntegerFromBytes(bytes, length)
      unsigned char * bytes;
      int length;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BIGNUM * a = BN_new();
+  BN_bin2bn(bytes, length, a);
+  return a;
+#elif defined(CRYPTOLIB)
   BigInteger rv, t;
   int i, n;
 
@@ -99,7 +110,7 @@ BigIntegerFromBytes(bytes, length)
     freeBignum(t);
   }
   return rv;
-#elif GNU_MP
+#elif defined(GNU_MP)
   BigInteger rv;
   char hexbuf[MAXHEXPARAMLEN];
   char *p;
@@ -117,14 +128,16 @@ BigIntegerToBytes(src, dest)
      BigInteger src;
      unsigned char * dest;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  return BN_bn2bin(src, dest);
+#elif defined(CRYPTOLIB)
   int i;
 
   trim(src);
   i = bigBytes(src);
   RSA_bigToBuf(src, i, dest);
   return i;
-#elif GNU_MP
+#elif defined(GNU_MP)
   char hexbuf[MAXHEXPARAMLEN];
   char * p;
   int r;
@@ -142,10 +155,12 @@ BigIntegerToHex(src, dest)
      BigInteger src;
      char * dest;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  strcpy(dest, BN_bn2hex(src));
+#elif defined(CRYPTOLIB)
   trim(src);
   bigsprint(src, dest);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_get_str(dest, 16, src);
 #endif
 }
@@ -154,9 +169,11 @@ int
 BigIntegerBitLen(b)
      BigInteger b;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  return BN_num_bits(b);
+#elif defined(CRYPTOLIB)
   return bigBits(b);
-#elif GNU_MP
+#elif defined(GNU_MP)
   return mpz_sizeinbase(b, 2);
 #endif
 }
@@ -165,9 +182,11 @@ int
 BigIntegerCmp(c1, c2)
      BigInteger c1, c2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  return BN_cmp(c1, c2);
+#elif defined(CRYPTOLIB)
   return bigCompare(c1, c2);
-#elif GNU_MP
+#elif defined(GNU_MP)
   return mpz_cmp(c1, c2);
 #endif
 }
@@ -177,7 +196,14 @@ BigIntegerCmpInt(c1, c2)
      BigInteger c1;
      unsigned int c2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BIGNUM * a = BN_new();
+  int rv;
+  BN_set_word(a, c2);
+  rv = BN_cmp(c1, a);
+  BN_free(a);
+  return rv;
+#elif defined(CRYPTOLIB)
   BigInteger t;
   int rv;
 
@@ -185,7 +211,7 @@ BigIntegerCmpInt(c1, c2)
   rv = bigCompare(c1, t);
   freeBignum(t);
   return rv;
-#elif GNU_MP
+#elif defined(GNU_MP)
   return mpz_cmp_ui(c1, c2);
 #endif
 }
@@ -195,9 +221,11 @@ BigIntegerLShift(result, x, bits)
      BigInteger result, x;
      unsigned int bits;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_lshift(result, x, bits);
+#elif defined(CRYPTOLIB)
   bigLeftShift(x, bits, result);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_mul_2exp(result, x, bits);
 #endif
 }
@@ -206,9 +234,11 @@ void
 BigIntegerAdd(result, a1, a2)
      BigInteger result, a1, a2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_add(result, a1, a2);
+#elif defined(CRYPTOLIB)
   bigAdd(a1, a2, result);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_add(result, a1, a2);
 #endif
 }
@@ -218,13 +248,18 @@ BigIntegerAddInt(result, a1, a2)
      BigInteger result, a1;
      unsigned int a2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BIGNUM * a = BN_new();
+  BN_set_word(a, a2);
+  BN_add(result, a1, a);
+  BN_free(a);
+#elif defined(CRYPTOLIB)
   BigInteger t;
 
   t = bigInit(a2);
   bigAdd(a1, t, result);
   freeBignum(t);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_add_ui(result, a1, a2);
 #endif
 }
@@ -233,9 +268,11 @@ void
 BigIntegerSub(result, s1, s2)
      BigInteger result, s1, s2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_sub(result, s1, s2);
+#elif defined(CRYPTOLIB)
   bigSubtract(s1, s2, result);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_sub(result, s1, s2);
 #endif
 }
@@ -245,13 +282,18 @@ BigIntegerSubInt(result, s1, s2)
      BigInteger result, s1;
      unsigned int s2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BIGNUM * s = BN_new();
+  BN_set_word(s, s2);
+  BN_sub(result, s1, s);
+  BN_free(s);
+#elif defined(CRYPTOLIB)
   BigInteger t;
 
   t = bigInit(s2);
   bigSubtract(s1, t, result);
   freeBignum(t);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_sub_ui(result, s1, s2);
 #endif
 }
@@ -260,9 +302,13 @@ void
 BigIntegerMul(result, m1, m2)
      BigInteger result, m1, m2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BN_mul(result, m1, m2, ctx);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
   bigMultiply(m1, m2, result);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_mul(result, m1, m2);
 #endif
 }
@@ -272,13 +318,19 @@ BigIntegerMulInt(result, m1, m2)
      BigInteger result, m1;
      unsigned int m2;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BIGNUM * m = BN_new();
+  BN_set_word(m, m2);
+  BN_mul(result, m1, m, ctx);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
   BigInteger t;
 
   t = bigInit(m2);
   bigMultiply(m1, t, result);
   freeBignum(t);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_mul_ui(result, m1, m2);
 #endif
 }
@@ -288,15 +340,25 @@ BigIntegerDivInt(result, d, m)
      BigInteger result, d;
      unsigned int m;
 {
-#ifdef CRYPTOLIB
-  BigInteger t, u;
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BIGNUM * a = BN_new();
+  BN_set_word(a, m);
+  BN_div(result, NULL, d, a, ctx);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
+  BigInteger t, u, q;
 
   t = bigInit(m);
   u = bigInit(0);
-  bigDivide(d, t, result, u);
+  /* We use a separate variable q because cryptolib breaks if result == d */
+  q = bigInit(0);
+  bigDivide(d, t, q, u);
   freeBignum(t);
   freeBignum(u);
-#elif GNU_MP
+  bigCopy(q, result);
+  freeBignum(q);
+#elif defined(GNU_MP)
 # ifdef GMP2
   mpz_fdiv_q_ui(result, d, m);
 # else
@@ -309,9 +371,13 @@ void
 BigIntegerMod(result, d, m)
      BigInteger result, d, m;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BN_mod(result, d, m, ctx);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
   bigMod(d, m, result);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_mod(result, d, m);
 #endif
 }
@@ -321,7 +387,9 @@ BigIntegerModInt(d, m)
      BigInteger d;
      unsigned int m;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  return BN_mod_word(d, m);
+#elif defined(CRYPTOLIB)
   BigInteger t, u;
   unsigned char r[4];
 
@@ -332,7 +400,7 @@ BigIntegerModInt(d, m)
   freeBignum(t);
   freeBignum(u);
   return r[0] | (r[1] << 8) | (r[2] << 16) | (r[3] << 24);
-#elif GNU_MP
+#elif defined(GNU_MP)
   MP_INT result;
   unsigned int i;
 
@@ -356,10 +424,14 @@ void
 BigIntegerModMul(r, m1, m2, modulus)
      BigInteger r, m1, m2, modulus;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BN_mod_mul(r, m1, m2, modulus, ctx);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
   bigMultiply(m1, m2, r);
   bigMod(r, modulus, r);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_mul(r, m1, m2);
   mpz_mod(r, r, modulus);
 #endif
@@ -369,9 +441,13 @@ void
 BigIntegerModExp(r, b, e, m)
      BigInteger r, b, e, m;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BN_mod_exp(r, b, e, m, ctx);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
   bigPow(b, e, m, r);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_powm(r, b, e, m);
 #endif
 }
@@ -382,13 +458,20 @@ BigIntegerModExpInt(r, b, e, m)
      unsigned int e;
      BigInteger m;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  BIGNUM * p = BN_new();
+  BN_set_word(p, e);
+  BN_mod_exp(r, b, p, m, ctx);
+  BN_free(p);
+  BN_CTX_free(ctx);
+#elif defined(CRYPTOLIB)
   BigInteger t;
 
   t = bigInit(e);
   bigPow(b, t, m, r);
   freeBignum(t);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_powm_ui(r, b, e, m);
 #endif
 }
@@ -397,9 +480,32 @@ int
 BigIntegerCheckPrime(n)
      BigInteger n;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_CTX * ctx = BN_CTX_new();
+  int rv = BN_is_prime(n, 25, NULL, ctx, NULL);
+  BN_CTX_free(ctx);
+  return rv;
+#elif defined(CRYPTOLIB)
+#if 0
+  /*
+   * Ugh.  Not only is cryptolib's bigDivide sensitive to inputs
+   * and outputs being the same, but now the primeTest needs random
+   * numbers, which it gets by calling cryptolib's broken truerand
+   * implementation(!)  We have to fake it out by doing our own
+   * seeding explicitly.
+   */
+  static int seeded = 0;
+  static unsigned char seedbuf[64];
+  if(!seeded) {
+    t_random(seedbuf, sizeof(seedbuf));
+    seedDesRandom(seedbuf, sizeof(seedbuf));
+    memset(seedbuf, 0, sizeof(seedbuf));
+    seeded = 1;
+  }
+#endif /* 0 */
+  t_random(NULL, 0);
   return primeTest(n);
-#elif GNU_MP
+#elif defined(GNU_MP)
   return mpz_probab_prime_p(n, 25);
 #endif
 }
@@ -408,9 +514,11 @@ void
 BigIntegerFree(b)
      BigInteger b;
 {
-#ifdef CRYPTOLIB
+#ifdef OPENSSL
+  BN_free(b);
+#elif defined(CRYPTOLIB)
   freeBignum(b);
-#elif GNU_MP
+#elif defined(GNU_MP)
   mpz_clear(b);
   free(b);
 #endif
