@@ -54,7 +54,20 @@
 
 #include "t_defines.h"
 
-#ifndef CRYPTOLIB
+#if defined(CRYPTOLIB) && defined(WIN32)
+
+/* Cryptolib contains its own truerand() on both UNIX and Windows. */
+/* Only use cryptolib's truerand under Windows */
+
+#include "libcrypt.h"
+
+unsigned long
+raw_truerand()
+{
+	return truerand();
+}
+
+#else /* CRYPTOLIB && WIN32 */
 
 #include <signal.h>
 #include <setjmp.h>
@@ -165,14 +178,4 @@ int n;
 	return v % n;
 }
 
-#else
-
-/* Cryptolib contains its own truerand() on both UNIX and Windows. */
-
-unsigned long
-raw_truerand()
-{
-	return truerand();
-}
-
-#endif
+#endif /* !CRYPTOLIB || !WIN32 */
