@@ -55,6 +55,9 @@ static char sccsid[] = "@(#)auth.c	8.3 (Berkeley) 5/30/95";
  * or implied warranty.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
 
 #if	defined(AUTHENTICATION)
 #include <stdio.h>
@@ -65,10 +68,10 @@ static char sccsid[] = "@(#)auth.c	8.3 (Berkeley) 5/30/95";
 #ifdef	__STDC__
 #include <stdlib.h>
 #endif
-#ifdef	NO_STRING_H
-#include <strings.h>
-#else
+#ifdef	HAVE_STRING_H
 #include <string.h>
+#else
+#include <strings.h>
 #endif
 
 #include "encrypt.h"
@@ -101,6 +104,16 @@ static	int	auth_send_last = -1;
  */
 Authenticator authenticators[] = {
 #ifdef	SRP
+#  ifdef TLS
+    { AUTHTYPE_SRP, AUTH_WHO_CLIENT|AUTH_HOW_ONE_WAY|AUTH_ENCRYPT_START_TLS,
+          srp_init,
+          srp_send,
+          srp_is,
+          srp_reply,
+          srp_status,
+          srp_printsub },
+#  endif /* TLS */
+#  ifdef ENCRYPTION
     { AUTHTYPE_SRP, AUTH_WHO_CLIENT|AUTH_HOW_ONE_WAY|AUTH_ENCRYPT_ON,
           srp_init,
           srp_send,
@@ -108,6 +121,7 @@ Authenticator authenticators[] = {
           srp_reply,
           srp_status,
           srp_printsub },
+#  endif /* ENCRYPTION */
     { AUTHTYPE_SRP, AUTH_WHO_CLIENT|AUTH_HOW_ONE_WAY,
 	  			srp_init,
 				srp_send,
@@ -133,6 +147,15 @@ Authenticator authenticators[] = {
 				spx_printsub },
 #endif
 #ifdef	KRB5
+#ifdef TLS
+    { AUTHTYPE_KERBEROS_V5, AUTH_WHO_CLIENT|AUTH_HOW_MUTUAL|AUTH_ENCRYPT_START_TLS,
+          kerberos5_init,
+          kerberos5_send,
+          kerberos5_is,
+          kerberos5_reply,
+          kerberos5_status,
+          kerberos5_printsub },
+#endif /* TLS */
 # ifdef	ENCRYPTION
     { AUTHTYPE_KERBEROS_V5, AUTH_WHO_CLIENT|AUTH_HOW_MUTUAL|AUTH_ENCRYPT_ON,
           kerberos5_init,
@@ -157,7 +180,7 @@ Authenticator authenticators[] = {
 				kerberos5_status,
 				kerberos5_printsub },
 #endif
-#ifdef	KRB4
+#if defined(KRB4) || defined(KRB4C)
 # ifdef ENCRYPTION
     { AUTHTYPE_KERBEROS_V4, AUTH_WHO_CLIENT|AUTH_HOW_MUTUAL|AUTH_ENCRYPT_ON,
           kerberos4_init,
@@ -190,6 +213,11 @@ static Authenticator NoAuth = { 0 };
 static int	i_support = 0;
 static int	i_wont_support = 0;
 
+#ifdef TLS
+extern int tls_active;
+extern int tls_anon;
+#endif
+
 	Authenticator *
 findauthenticator(type, way)
 	int type;
@@ -356,13 +384,26 @@ auth_request()
 	static unsigned char str_request[64] = { IAC, SB,
 						 TELOPT_AUTHENTICATION,
 						 TELQUAL_SEND, };
-	Authenticator *ap = authenticators;
+	Authenticator *ap;
 	unsigned char *e = str_request + 4;
 
 	if (!authenticating) {
 		authenticating = 1;
-		while (ap->type) {
+		for (ap = authenticators; ap->type; ++ap) {
 			if (i_support & ~i_wont_support & typemask(ap->type)) {
+				/* Filter out ways that don't make sense */
+#ifdef TLS
+				if ((ap->way & AUTH_ENCRYPT_MASK) ==
+				    AUTH_ENCRYPT_USING_TELOPT && tls_active)
+				  continue;	/* Don't ENCRYPT inside TLS */
+				if ((ap->way & AUTH_ENCRYPT_MASK) ==
+				    AUTH_ENCRYPT_START_TLS && !tls_active)
+				  continue;  /* ENCRYPT_START_TLS needs TLS */
+				if ((ap->way & AUTH_ENCRYPT_MASK) !=
+				    AUTH_ENCRYPT_START_TLS &&
+				    tls_active && tls_anon)
+				  continue;  /* ANON needs ENCRYPT_START_TLS */
+#endif
 				if (auth_debug_mode) {
 					printf(">>>%s: Sending type %d %d\r\n",
 						Name, ap->type, ap->way);
@@ -370,7 +411,6 @@ auth_request()
 				*e++ = ap->type;
 				*e++ = ap->way;
 			}
-			++ap;
 		}
 		*e++ = IAC;
 		*e++ = SE;
@@ -445,6 +485,17 @@ auth_send(data, cnt)
 
             if ((ap->way & AUTH_ENCRYPT_MASK) && !auth_enable_encrypt)
                 continue;
+#ifdef TLS	/* Skip inappropriate ENCRYPT proposals */
+	    if ((ap->way & AUTH_ENCRYPT_MASK) == AUTH_ENCRYPT_USING_TELOPT &&
+		tls_active)
+	      continue;		/* Don't ENCRYPT inside TLS */
+	    if ((ap->way & AUTH_ENCRYPT_MASK) == AUTH_ENCRYPT_START_TLS &&
+		!tls_active)
+	      continue;		/* Can't do ENCRYPT_START_TLS without TLS */
+	    if ((ap->way & AUTH_ENCRYPT_MASK) != AUTH_ENCRYPT_START_TLS &&
+		tls_active && tls_anon)
+	      continue;		/* Must have ENCRYPT_START_TLS if anonymous */
+#endif
 
 	    /* This prevents us from re-using the same failed method */
 	    if(auth_send_data[0] == auth_send_last)
@@ -474,6 +525,14 @@ auth_send(data, cnt)
 		printf(">>>%s: Sent failure message\r\n", Name);
 	auth_finished(0, AUTH_REJECT);
         auth_has_failed = 1;
+	auth_warn();
+}
+
+void
+auth_warn()
+{
+  char rbuf[64];
+
 #ifdef KANNAN
 	/*
 	 *  We requested strong authentication, however no mechanisms worked.
@@ -483,7 +542,24 @@ auth_send(data, cnt)
 	exit(0);
 #endif /* KANNAN */
 #ifdef TJW
-	printf("WARNING: unable to authenticate securely; falling back to standard login.\n");
+#  ifdef TLS
+	if(tls_active) {
+	  /*
+	   * If we're using TLS anonymously, warn the user.  If we're not
+	   * anonymous, we're at least using server authentication, and maybe
+	   * even client-side authentication, so we'll be quiet and let
+	   * the user trust the PKI.
+	   */
+	  if(tls_anon) {
+	    printf("WARNING: unable to verify TLS session parameters.  ");
+	    read_string(rbuf, sizeof(rbuf) - 1, "Continue? (Y/N) ");
+	    if(!(rbuf[0] == 'y' || rbuf[0] == 'Y'))
+	      clean_exit(1);
+	  }
+	}
+	else
+#  endif
+	printf("WARNING: unable to authenticate securely; falling back to standard login.\r\n");
 #endif /* TJW */
 }
 
@@ -549,6 +625,7 @@ auth_name(data, cnt)
 	if (cnt < 1) {
 		if (auth_debug_mode)
 			printf(">>>%s: Empty name in NAME\r\n", Name);
+                auth_encrypt_user(NULL);
 		return;
 	}
 	if (cnt > sizeof(savename) - 1) {
@@ -612,8 +689,13 @@ auth_wait(name)
 	if (auth_debug_mode)
 		printf(">>>%s: in auth_wait.\r\n", Name);
 
+        /* 
+         * The authenticating flag is only set if we are a server.
+         * auth_wait() is only called by the client if we are in
+         * fact authenticating, so the flag is ignored 
+         */
 	if (Server && !authenticating)
-		return(0);
+		return(AUTH_REJECT);
 
 	(void) signal(SIGALRM, auth_intr);
 	alarm(300);
@@ -623,12 +705,22 @@ auth_wait(name)
 	alarm(0);
 	(void) signal(SIGALRM, SIG_DFL);
 
+        if (auth_debug_mode)
+                printf(">>>%s: leaving auth_wait().\r\n", Name);
+
 	/*
 	 * Now check to see if the user is valid or not
 	 */
 	if (!authenticated || authenticated == &NoAuth)
 		return(AUTH_REJECT);
 
+        /*
+         * auth_wait() returns from here if we are a client. 
+         * authenticated->status is only for servers.
+         */
+        if (!Server) 
+                return(validuser = AUTH_OTHER);
+
 	if (validuser == AUTH_VALID)
 		validuser = AUTH_USER;
 
