@@ -30,16 +30,33 @@
  * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
  * SUCH DAMAGE.
  */
+/*
+ * START_TLS and FORWARD_X support from Peter Runestig <peter@runestig.com>
+ * and Jeffrey Altman <jaltman@columbia.edu> incorporated into the
+ * SRP Telnet distribution 10/19/2000 by Tom Wu <tjw@cs.stanford.edu>.
+ */
 
 #ifndef lint
 static char sccsid[] = "@(#)state.c	8.5 (Berkeley) 5/30/95";
 #endif /* not lint */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "telnetd.h"
 #if	defined(AUTHENTICATION)
 #include <libtelnet/auth.h>
 #endif
 
+#ifdef FWD_X
+#include <sys/socket.h>
+#include <netinet/in.h>
+#include <arpa/inet.h>
+#include <sys/time.h>
+#include "fwdxutil.h"
+#endif
+
 unsigned char	doopt[] = { IAC, DO, '%', 'c', 0 };
 unsigned char	dont[] = { IAC, DONT, '%', 'c', 0 };
 unsigned char	will[] = { IAC, WILL, '%', 'c', 0 };
@@ -51,7 +68,7 @@ static int envvarok P((char *));
  * Buffer for sub-options, and macros
  * for suboptions buffer manipulations
  */
-unsigned char subbuffer[2048], *subpointer= subbuffer, *subend= subbuffer;
+unsigned char subbuffer[4096], *subpointer= subbuffer, *subend= subbuffer;
 
 #define	SB_CLEAR()	subpointer = subbuffer
 #define	SB_TERM()	{ subend = subpointer; SB_CLEAR(); }
@@ -93,7 +110,7 @@ static void sb_auth_complete()
   }
 }
 
-	void
+void
 telrcv()
 {
 	register int c;
@@ -101,9 +118,12 @@ telrcv()
 #if	defined(CRAY2) && defined(UNICOS5)
 	char *opfrontp = pfrontp;
 #endif
+#ifdef FWD_X
+	unsigned char *subroof;
+#endif
 
 	while (ncc > 0) {
-		if ((&ptyobuf[BUFSIZ] - pfrontp) < 2)
+		if ((&ptyobuf[BUFSIZ] - pfrontp) < 1)
 			break;
 		c = *netip++ & 0377, ncc--;
 #ifdef	ENCRYPTION
@@ -306,6 +326,20 @@ gotiac:			switch (c) {
 			if (c == IAC) {
 				state = TS_SE;
 			} else {
+#ifdef FWD_X
+				/* flush the subbuffer if it is going to overflow */
+				subroof = subbuffer + sizeof(subbuffer) - 3;
+				if ((subpointer >= subroof) && subbuffer[0] == TELOPT_FORWARD_X &&
+					subbuffer[1] == FWDX_DATA) {
+				    int channel = (subbuffer[2] << 8) + subbuffer[3];
+				    fwdx_redirect(channel, subbuffer + 4, subpointer - subbuffer - 4);
+				    /* append IAC SE just for the sake of DIAG */
+				    subpointer[0] = IAC; subpointer[1] = SE;
+	    			    DIAG(TD_OPTIONS, printsub('<', subbuffer, subpointer - subbuffer + 2););
+		    		/* start filling subbuffer again after the "channel" bytes */
+		    		subpointer = subbuffer + 4;
+				}
+#endif
 				SB_ACCUM(c);
 			}
 			break;
@@ -332,6 +366,19 @@ gotiac:			switch (c) {
 					state = TS_IAC;
 					goto gotiac;
 				}
+#ifdef FWD_X
+				/* flush the subbuffer if it is going to overflow */
+				subroof = subbuffer + sizeof(subbuffer) - 3;
+				if ((subpointer >= subroof) && subbuffer[0] == TELOPT_FORWARD_X &&
+					subbuffer[1] == FWDX_DATA) {
+				    int channel = (subbuffer[2] << 8) + subbuffer[3];
+	    	    		    fwdx_redirect(channel, subbuffer + 4, subpointer - subbuffer - 4);
+				    subpointer[0] = IAC; subpointer[1] = SE;
+	    			    DIAG(TD_OPTIONS, printsub('<', subbuffer, subpointer - subbuffer + 2););
+		    		/* start filling subbuffer again after the "channel" bytes */
+		    		subpointer = subbuffer + 4;
+				}
+#endif
 				SB_ACCUM(c);
 				state = TS_SB;
 			} else {
@@ -367,9 +414,9 @@ gotiac:			switch (c) {
 			continue;
 
 		default:
-			syslog(LOG_ERR, "telnetd: panic state=%d\n", state);
+			syslog(LOG_ERR, "telnetd: panic state=%d", state);
 			printf("telnetd: panic state=%d\n", state);
-			exit(1);
+			clean_exit(1);
 		}
 	}
 #if	defined(CRAY2) && defined(UNICOS5)
@@ -445,7 +492,7 @@ gotiac:			switch (c) {
  * is complete.
  *
  */
-	void
+void
 send_do(option, init)
 	int option, init;
 {
@@ -464,7 +511,7 @@ send_do(option, init)
 			set_his_want_state_will(option);
 		do_dont_resp[option]++;
 	}
-	(void) sprintf(nfrontp, (char *)doopt, option);
+	(void) net_printf((char *)doopt, option);
 	nfrontp += sizeof (dont) - 2;
 
 	DIAG(TD_OPTIONS, printoption("td: send do", option));
@@ -480,7 +527,7 @@ extern void doclientstat();
 extern void encrypt_send_support();
 #endif	/* ENCRYPTION */
 
-	void
+void
 willoption(option)
 	int option;
 {
@@ -587,6 +634,8 @@ willoption(option)
 
 #ifdef	AUTHENTICATION
 		case TELOPT_AUTHENTICATION:
+			if(auth_ignore)
+				break;
 			func = auth_request;
 			changeok++;
 			break;
@@ -594,6 +643,10 @@ willoption(option)
 
 #ifdef	ENCRYPTION
 		case TELOPT_ENCRYPT:
+#  ifdef TLS
+			if(tls_active)
+				break;
+#  endif
 			func = encrypt_send_support;
 			changeok++;
 			break;
@@ -653,12 +706,18 @@ willoption(option)
 
 #ifdef	AUTHENTICATION
 		case TELOPT_AUTHENTICATION:
+			if(auth_ignore)
+				break;
 			func = auth_request;
 			break;
 #endif
 
 #ifdef	ENCRYPTION
 		case TELOPT_ENCRYPT:
+#  ifdef TLS
+			if(tls_active)
+				break;
+#  endif
 			func = encrypt_send_support;
 			break;
 #endif	/* ENCRYPTION */
@@ -684,7 +743,7 @@ send_dont(option, init)
 		set_his_want_state_wont(option);
 		do_dont_resp[option]++;
 	}
-	(void) sprintf(nfrontp, (char *)dont, option);
+	(void) net_printf((char *)dont, option);
 	nfrontp += sizeof (doopt) - 2;
 
 	DIAG(TD_OPTIONS, printoption("td: send dont", option));
@@ -834,7 +893,7 @@ send_will(option, init)
 		set_my_want_state_will(option);
 		will_wont_resp[option]++;
 	}
-	(void) sprintf(nfrontp, (char *)will, option);
+	(void) net_printf((char *)will, option);
 	nfrontp += sizeof (doopt) - 2;
 
 	DIAG(TD_OPTIONS, printoption("td: send will", option));
@@ -955,9 +1014,23 @@ dooption(option)
 
 #ifdef	ENCRYPTION
 		case TELOPT_ENCRYPT:
+#  ifdef TLS
+			if(tls_active)
+				break;
+#  endif
 			changeok++;
 			break;
 #endif	/* ENCRYPTION */
+#ifdef	FWD_X
+		case TELOPT_FORWARD_X:
+			if (!fwdx_disable_flag) {
+				if (!fwdx_sbdata)
+					fwdx_sbdata = malloc(fwdx_sbdata_size);
+				if (fwdx_sbdata && encrypt_is_encrypting())
+					changeok++;
+			}
+			break;
+#endif	/* FWD_X */
 		case TELOPT_LINEMODE:
 		case TELOPT_TTYPE:
 		case TELOPT_NAWS:
@@ -980,10 +1053,15 @@ dooption(option)
 		}
 	}
 	set_my_state_will(option);
-
+#ifdef FWD_X
+	/* if fwdx_sbdata == NULL, we didn't send WILL FORWARD_X */
+        if (option == TELOPT_FORWARD_X && fwdx_sbdata &&
+             my_want_state_is_will(option))
+                fwdx_send_options();
+#endif
 }  /* end of dooption */
 
-	void
+void
 send_wont(option, init)
 	int option, init;
 {
@@ -994,13 +1072,13 @@ send_wont(option, init)
 		set_my_want_state_wont(option);
 		will_wont_resp[option]++;
 	}
-	(void) sprintf(nfrontp, (char *)wont, option);
+	(void) net_printf((char *)wont, option);
 	nfrontp += sizeof (wont) - 2;
 
 	DIAG(TD_OPTIONS, printoption("td: send wont", option));
 }
 
-	void
+void
 dontoption(option)
 	int option;
 {
@@ -1072,6 +1150,11 @@ dontoption(option)
 				send_will(option, 1);
 			return;
 #endif	/* defined(LINEMODE) && defined(KLUDGELINEMODE) */
+#ifdef FWD_X
+                case TELOPT_FORWARD_X:
+                        fwdx_cleanup();
+                        break;
+#endif /* FWD_X */
 
 		default:
 			break;
@@ -1106,7 +1189,7 @@ int env_ovalue = -1;
  *	Window size
  *	Terminal speed
  */
-	void
+void
 suboption()
 {
     register int subchar;
@@ -1269,6 +1352,9 @@ suboption()
         sb_auth_complete();
 	settimer(xdisplocsubopt);
 	subpointer[SB_LEN()] = '\0';
+#ifdef FWD_X
+	if (fwdx_listen_sock < 0)
+#endif
 	(void)setenv("DISPLAY", (char *)subpointer, 1);
 	break;
     }  /* end of case TELOPT_XDISPLOC */
@@ -1396,8 +1482,8 @@ suboption()
 	    env_ovar_wrong:
 			env_ovar = OLD_ENV_VALUE;
 			env_ovalue = OLD_ENV_VAR;
-			DIAG(TD_OPTIONS, {sprintf(nfrontp,
-				"ENVIRON VALUE and VAR are reversed!\r\n");
+			DIAG(TD_OPTIONS,
+			    {net_printf("ENVIRON VALUE and VAR are reversed!\r\n");
 				nfrontp += strlen(nfrontp);});
 
 		}
@@ -1436,12 +1522,22 @@ suboption()
 		case NEW_ENV_VAR:
 		case ENV_USERVAR:
 			*cp = '\0';
+#ifdef FWD_X
+			/* if FWD_X is on, we can't propagate DISPLAY or XAUTHORITY */
+			if (!(valp && 
+                            my_state_is_will(TELOPT_FORWARD_X) &&
+                            (!strcmp(varp, "DISPLAY") ||
+			     !strcmp(varp, "XAUTHORITY")))) {
+#endif
                     if (envvarok(varp)) {
 			if (valp)
 				(void)setenv(varp, valp, 1);
 			else
 				unsetenv(varp);
                         }
+#ifdef FWD_X
+			}	
+#endif
 			cp = varp = (char *)subpointer;
 			valp = 0;
 			break;
@@ -1457,12 +1553,22 @@ suboption()
 		}
 	}
 	*cp = '\0';
+#ifdef FWD_X
+	/* if FWD_X is on, we can't propagate DISPLAY or XAUTHORITY */
+	if (!(valp && 
+	      my_state_is_will(TELOPT_FORWARD_X) &&
+	      (!strcmp(varp, "DISPLAY") ||
+	       !strcmp(varp, "XAUTHORITY")))) {
+#endif
         if (envvarok(varp)) {
 	if (valp)
 		(void)setenv(varp, valp, 1);
 	else
 		unsetenv(varp);
         }
+#ifdef FWD_X
+	}	
+#endif
 	break;
     }  /* end of case TELOPT_NEW_ENVIRON */
 #if	defined(AUTHENTICATION)
@@ -1490,6 +1596,9 @@ suboption()
 #endif
 #ifdef	ENCRYPTION
     case TELOPT_ENCRYPT:
+	if (his_state_is_wont(TELOPT_ENCRYPT) &&
+            his_state_is_dont(TELOPT_ENCRYPT))	/* Ignore if option disabled */
+		break;
 	if (SB_EOF())
 		break;
 	switch(SB_GET()) {
@@ -1532,13 +1641,76 @@ suboption()
 	break;
 #endif	/* ENCRYPTION */
 
+#ifdef TLS
+    case TELOPT_START_TLS:
+    	if (SB_EOF())
+	    break;
+        switch (SB_GET()) {
+	    case TLS_FOLLOWS:
+		tls_follows_from_client = 1;
+	        break;
+	    default:
+	        break;
+	}
+        break;
+#endif  /* TLS */
+
+#ifdef FWD_X
+    case TELOPT_FORWARD_X:
+    	if (!fwdx_sbdata || SB_EOF()) /* if fwdx_sbdata == NULL here, something is wrong, this */
+	    break;                    /* suboption is unexpected and we should not continue.   */
+        switch (SB_GET()) {
+	    case FWDX_SCREEN: {
+	    	int dn, sn;
+    		if (SB_EOF())
+	    	    break;
+		sn = SB_GET(); /* sn = screen number from the subneg */
+	    	if (fwdx_sbdata && (dn = fwdx_listen()) >= 0) {
+                    if (fwdx_setup_xauth())
+                        send_wont(TELOPT_FORWARD_X, 0);    
+		}
+		else
+		    send_wont(TELOPT_FORWARD_X, 0);    
+		break; }
+	    case FWDX_CLOSE: {
+		int channel;
+	    	if (SB_LEN() < 2)
+		    break;
+		channel = (subpointer[0] << 8) + subpointer[1];
+		fwdx_close_channel(channel);
+	        break; }
+	    case FWDX_DATA: {
+		int channel;
+	    	if (SB_LEN() < 3)
+		    break;
+		channel = (subpointer[0] << 8) + subpointer[1];
+	    	fwdx_redirect(channel, subpointer + 2, SB_LEN() - 2);
+	        break; }
+	    case FWDX_OPTIONS:
+    		if (SB_EOF())
+	    	    break;
+	    	fwdx_do_client_options(subpointer, SB_LEN());
+	        break;
+	    case FWDX_OPT_DATA:
+	    	if (SB_LEN() < 2)
+		    break;
+		switch (SB_GET()) {
+		    default:
+		    	break;
+		}
+	    default:
+	        break;
+	}
+    	break;
+#endif  /* FWD_X */
+
     default:
 	break;
     }  /* end of switch */
 
 }  /* end of suboption */
 
-	void
+void
 doclientstat()
 {
 	clientstat(TELOPT_LINEMODE, WILL, 0);
@@ -1546,7 +1718,7 @@ doclientstat()
 
 #define	ADD(c)	 *ncp++ = c
 #define	ADD_DATA(c) { *ncp++ = c; if (c == SE || c == IAC) *ncp++ = c; }
-	void
+void
 send_status()
 {
 	unsigned char statusbuf[256];
@@ -1604,7 +1776,7 @@ send_status()
 				ADD(LFLOW_RESTART_XON);
 			}
 			ADD(SE);
-                        ADD(SB);
+                        /*ADD(SB);*/
 		}
 	}
 
@@ -1666,6 +1838,9 @@ static int envvarok(varp)
 	    strcmp(varp, "KRB_REALMS") &&  /* cns v4 */
 	    strcmp(varp, "LIBPATH") &&     /* AIX */
 	    strcmp(varp, "RESOLV_HOST_CONF") && /* linux */
+	    strcmp(varp, "SHLIB_PATH") &&
+	    strcmp(varp, "ENV") &&
+	    strcmp(varp, "BASH_ENV") &&
 	    strcmp(varp, "IFS")) {
 		return 1;
 	} else {
