@@ -47,6 +47,11 @@ you didn't get a copy, you may request one from <license@inner.net>.
  * IPv6+IPsec modifications brought up to date for telnet.98.02.16 by
  * Albert K T Hui <avatar@deva.net>.  All rights reserved.
  */
+/*
+ * START_TLS and FORWARD_X support from Peter Runestig <peter@runestig.com>
+ * and Jeffrey Altman <jaltman@columbia.edu> incorporated into the
+ * SRP Telnet distribution 10/19/2000 by Tom Wu <tjw@cs.stanford.edu>.
+ */
 
 #ifndef lint
 static char copyright[] =
@@ -58,6 +63,18 @@ static char copyright[] =
 static char sccsid[] = "@(#)telnetd.c	8.4 (Berkeley) 5/30/95";
 #endif /* not lint */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef TLS
+#define RECV tls_recv
+#define READ tls_read
+#else
+#define RECV recv
+#define READ read
+#endif
+
 #include "telnetd.h"
 #include "pathnames.h"
 
@@ -93,9 +110,12 @@ struct	socket_security ss;
 # endif /* SO_SEC_MULTI */
 #endif	/* _SC_CRAY_SECURE_SYS */
 
+#ifdef KRB5
+#include "krb5.h"
+#endif
+
 #if	defined(AUTHENTICATION)
 #include <libtelnet/auth.h>
-int	auth_level = 0;
 #endif
 #if	defined(SecurID)
 int	require_SecurID = 0;
@@ -104,16 +124,21 @@ int	require_SecurID = 0;
 extern	int utmp_len;
 int	registerd_host_only = 0;
 
+#ifdef HAVE_SYS_SPT_H
+#include <sys/spt.h>
+#endif /* HAVE_SYS_SPT_H */
+
 #ifdef	STREAMSPTY
+# include <sys/stream.h>
 # include <stropts.h>
 # include <termio.h>
-#if defined(IRIX) || defined(OSF)
-# include <sys/stream.h>
-#else
+# ifdef HAVE_SYS_TTY_H
 /* make sure we don't get the bsd version */
-# include "/usr/include/sys/tty.h"
-# include <sys/ptyvar.h>
-#endif
+#  include "/usr/include/sys/tty.h"
+# endif
+# ifdef HAVE_SYS_PTYVAR_H
+#  include <sys/ptyvar.h>
+# endif
 
 /*
  * Because of the way ptyibuf is used with streams messages, we need
@@ -151,6 +176,10 @@ int debug = 0;
 int keepalive = 1;
 char *progname;
 
+int maxhostlen = 0;
+int always_ip = 0;
+int stripdomain = 1;
+
 extern void usage P((void));
 
 /*
@@ -159,7 +188,7 @@ extern void usage P((void));
  * passed off to getopt().
  */
 char valid_opts[] = {
-	'd', ':', 'h', 'k', 'n', 'S', ':', 'u', ':', 'U',
+	'd', ':', 'h', 'k', 'L', ':', 'n', 'S', ':', 'u', ':', 'U', 'w',
 #ifdef	AUTHENTICATION
 	'a', ':', 'X', ':',
 #endif
@@ -183,10 +212,20 @@ char valid_opts[] = {
 #endif
 #ifdef	SecurID
 	's',
+#endif
+#ifdef KRB5
+	'R', ':', 't', ':',
+#endif
+#ifdef TLS
+	'T', ':',
+#endif
+#ifdef FWD_X
+	'f', ':', 'F',
 #endif
 	'\0'
 };
 
+int
 main(argc, argv)
 	char *argv[];
 {
@@ -205,10 +244,16 @@ main(argc, argv)
 
 	pfrontp = pbackp = ptyobuf;
 	netip = netibuf;
+	netobuf_size = 2 * BUFSIZ + NETSLOP;
+	netobuf = malloc(netobuf_size);
+	if (!netobuf)
+	    return 1;
 	nfrontp = nbackp = netobuf;
 #ifdef	ENCRYPTION
 	nclearto = 0;
 #endif	/* ENCRYPTION */
+	login_uid = -1;
+	login_gid = -1;
 
 	progname = *argv;
 
@@ -219,6 +264,9 @@ main(argc, argv)
 	 */
 	highpty = getnpty();
 #endif /* CRAY */
+#ifdef TLS
+	tls_set_defaults();
+#endif
 
 	while ((ch = getopt(argc, argv, valid_opts)) != EOF) {
 		switch(ch) {
@@ -327,6 +375,14 @@ main(argc, argv)
 #endif	/* defined(LINEMODE) && defined(KLUDGELINEMODE) */
 			break;
 
+		case 'L':
+		    {
+		        extern char *login_program;
+
+			login_program = optarg;
+			break;
+		    }
+
 		case 'n':
 			keepalive = 0;
 			break;
@@ -359,6 +415,25 @@ main(argc, argv)
 		    }
 #endif	/* CRAY */
 
+#ifdef KRB5
+		case 'R':
+		    {
+			extern krb5_context telnet_context;
+			krb5_error_code retval;
+			
+			if (telnet_context == 0) {
+				retval = krb5_init_context(&telnet_context);
+				if (retval) {
+					com_err("telnetd", retval,
+						"while initializing krb5");
+					exit(1);
+				}
+			}
+			krb5_set_default_realm(telnet_context, optarg);
+			break;
+		    }
+#endif	/* KRB5 */
+
 #ifdef	SecurID
 		case 's':
 			/* SecurID required */
@@ -377,6 +452,16 @@ main(argc, argv)
 #endif
 			break;
 
+#ifdef KRB5
+		case 't':
+		    {
+			extern char *telnet_srvtab;
+
+			telnet_srvtab = optarg;
+			break;
+		    }
+#endif	/* KRB5 */
+
 		case 'u':
 			utmp_len = atoi(optarg);
 			break;
@@ -393,6 +478,42 @@ main(argc, argv)
 			auth_disable_name(optarg);
 			break;
 #endif	/* AUTHENTICATION */
+		case 'w':
+			if (!strcmp(optarg, "ip"))
+				always_ip = 1;
+			else {
+				char *cp;
+				cp = strchr(optarg, ',');
+				if (cp == NULL)
+					maxhostlen = atoi(optarg);
+				else if (*(++cp)) {
+					if (!strcmp(cp, "striplocal"))
+						stripdomain = 1;
+					else if (!strcmp(cp, "nostriplocal"))
+						stripdomain = 0;
+					else {
+						usage();
+					}
+					*(--cp) = '\0';
+					maxhostlen = atoi(optarg);
+				}
+			}
+			break;
+
+#ifdef TLS
+		case 'T':
+			tls_optarg(optarg);
+			break;
+#endif  /* TLS */
+
+#ifdef FWD_X
+		case 'F':
+			fwdx_disable_flag = 1;
+			break;
+		case 'f':
+			fwdx_disable_xauth_type(optarg);
+			break;
+#endif  /* FWD_X */
 
 		default:
 			fprintf(stderr, "telnetd: %c: unknown option\n", ch);
@@ -490,7 +611,7 @@ main(argc, argv)
 		sp = getservbyname("telnet", "tcp");
 		if (sp == 0) {
 		    fprintf(stderr, "telnetd: tcp/telnet: unknown service\n");
-		    exit(1);
+		    clean_exit(1);
 		}
 		sin.sin_port = sp->s_port;
 	    }
@@ -498,23 +619,23 @@ main(argc, argv)
 	    s = socket(AF_INET, SOCK_STREAM, 0);
 	    if (s < 0) {
 		    perror("telnetd: socket");;
-		    exit(1);
+		    clean_exit(1);
 	    }
 	    (void) setsockopt(s, SOL_SOCKET, SO_REUSEADDR,
 				(char *)&on, sizeof(on));
 	    if (bind(s, (struct sockaddr *)&sin, sizeof sin) < 0) {
 		perror("bind");
-		exit(1);
+		clean_exit(1);
 	    }
 	    if (listen(s, 1) < 0) {
 		perror("listen");
-		exit(1);
+		clean_exit(1);
 	    }
 	    foo = sizeof sin;
 	    ns = accept(s, (struct sockaddr *)&sin, &foo);
 	    if (ns < 0) {
 		perror("accept");
-		exit(1);
+		clean_exit(1);
 	    }
 	    (void) dup2(ns, 0);
 	    (void) close(ns);
@@ -546,7 +667,7 @@ main(argc, argv)
 
 		if (getsysv(&sysv, sizeof(struct sysv)) != 0) {
 			perror("getsysv");
-			exit(1);
+			clean_exit(1);
 		}
 
 		/*
@@ -559,7 +680,7 @@ main(argc, argv)
 		    (getsockopt(0, SOL_SOCKET, SO_SEC_MULTI,
 				(char *)&sock_multi, &szi) < 0)) {
 			perror("getsockopt");
-			exit(1);
+			clean_exit(1);
 		} else {
 			dv.dv_actlvl = ss.ss_actlabel.lt_level;
 			dv.dv_actcmp = ss.ss_actlabel.lt_compart;
@@ -591,7 +712,7 @@ main(argc, argv)
 	if (getpeername(0, (struct sockaddr *)&from, &fromlen) < 0) {
 		fprintf(stderr, "%s: ", progname);
 		perror("getpeername");
-		_exit(1);
+		clean_exit(1);
 	}
 	if (keepalive &&
 	    setsockopt(0, SOL_SOCKET, SO_KEEPALIVE,
@@ -616,6 +737,14 @@ main(argc, argv)
 	}
 #endif	/* defined(IPPROTO_IP) && defined(IP_TOS) */
 	net = 0;
+#ifdef FWD_X
+	fwdx_init();
+#endif
+#ifdef TLS
+	if (tls_init()) {
+		clean_exit(1);
+	}
+#endif
 #ifdef INET6
         doit(&from, fromlen);
 #else /* INET6 */
@@ -655,7 +784,6 @@ usage()
 #ifdef	CRAY
 	fprintf(stderr, " [-r[lowpty]-[highpty]]");
 #endif
-	fprintf(stderr, "\n\t");
 #ifdef	SecurID
 	fprintf(stderr, " [-s]");
 #endif
@@ -665,9 +793,18 @@ usage()
 #ifdef	AUTHENTICATION
 	fprintf(stderr, " [-X auth-type]");
 #endif
-	fprintf(stderr, " [-u utmp_hostname_length] [-U]");
+	fprintf(stderr, "\n\t");
+#ifdef TLS
+	fprintf(stderr, " [-T tls-required] [-T {rsa|dsa}cert=FILE] [-T {rsa|dsa}key=FILE]\n\t");
+	fprintf(stderr,
+	    " [-T crlfile=FILE] [-T crldir=DIR] [-T cipher=STRING] [-T dhparam=FILE]\n\t");
+#endif
+#ifdef FWD_X
+	fprintf(stderr, " [-F] [-f XAUTH-TYPE]\n\t");
+#endif
+	fprintf(stderr, " [-L login_prog] [-u utmp_hostname_length] [-U]");
 	fprintf(stderr, " [port]\n");
-	exit(1);
+	clean_exit(1);
 }
 
 static void encrypt_failure()
@@ -675,11 +812,49 @@ static void encrypt_failure()
     char *error_message =
 	"Encryption was not successfully negotiated.  Goodbye.\r\n\r\n";
 
+    fprintf(stderr,error_message);
     writenet(error_message, strlen(error_message));
     netflush();
     exit(1);
 }
 
+/*
+ * telnetd uid/gid callback routines
+ * Author: Tom Wu (tjw@CS.Stanford.EDU)
+ * These routines handle the process of re-assigning the uid/gid
+ * of files created by telnetd to those of the logged-in user.
+ * probe_uid_needed() returns non-zero if any files require
+ * an ownership change, and do_uid_callbacks() is called with
+ * the uid/gid of the logged-in user, once that information is
+ * available, and it is responsible for calling any option-specific
+ * routines that do the chown()ing.
+ */
+int
+probe_uid_needed()
+{
+#ifdef FWD_X
+  if(fwdx_xauthfile != NULL)
+    return 1;
+#endif /* FWD_X */
+#ifdef FORWARD
+  if(forw_credfile_exists())
+    return 1;
+#endif /* FORWARD */
+  return 0;
+}
+
+void
+do_uid_callbacks(uid, gid)
+     int uid, gid;
+{
+#ifdef FWD_X
+  fwdx_set_xauthfile_owner(uid, gid);
+#endif /* FWD_X */
+#ifdef FORWARD
+  forw_set_credfile_owner(uid, gid);
+#endif /* FORWARD */
+}
+
 /*
  * getterminaltype
  *
@@ -702,64 +877,108 @@ getterminaltype(name)
     /*
      * Handle the Authentication option before we do anything else.
      */
-    send_do(TELOPT_AUTHENTICATION, 1);
-    while (his_will_wont_is_changing(TELOPT_AUTHENTICATION))
+    if(!auth_ignore) {
+      send_do(TELOPT_AUTHENTICATION, 1);
+      while (his_will_wont_is_changing(TELOPT_AUTHENTICATION))
 	ttloop();
-    if (his_state_is_will(TELOPT_AUTHENTICATION)) {
+      if (his_state_is_will(TELOPT_AUTHENTICATION)) {
 	retval = auth_wait(name);
+      }
     }
 #endif
 
 #ifdef	ENCRYPTION
+#  ifdef TLS
+    if (!tls_active) {		/* Don't use TELOPT ENCRYPT under TLS */
+#  endif
     send_will(TELOPT_ENCRYPT, 1);
     send_do(TELOPT_ENCRYPT, 1);
+    while (his_do_dont_is_changing(TELOPT_ENCRYPT) ||
+           his_will_wont_is_changing(TELOPT_ENCRYPT))
+        ttloop();
+
+        /*
+        * Wait for the negotiation of what type of encryption we can
+        * send with.  If autoencrypt is not set, this will just return.
+        */
+        if (his_state_is_will(TELOPT_ENCRYPT)) {
+            encrypt_wait();
+        }
+        if (his_state_is_do(TELOPT_ENCRYPT)) {
+            decrypt_wait();
+        }
+
+        if (auth_must_encrypt()) {
+            time_t timeout = time(0) + 60;
+	
+            if (my_state_is_dont(TELOPT_ENCRYPT) ||
+                 my_state_is_wont(TELOPT_ENCRYPT)) {
+                fprintf(stderr,"my state wont/dont encrypt\r\n");
+                encrypt_failure();
+            }
+
+            while (!encrypt_is_encrypting()) {
+                if (time(0) > timeout) {
+                    fprintf(stderr,"Encrypt timeout\r\n");
+                    encrypt_failure();
+                }
+                ttloop();
+            }
+
+            if (!EncryptStartInput()) {
+                fprintf(stderr,"Not Encrypt Start I\r\n");
+                encrypt_failure();
+            }
+
+            if (!EncryptStartOutput()) {
+                fprintf(stderr,"Not Encrypt Start O\r\n");
+                encrypt_failure();
+            }
+
+        }
+#  ifdef TLS
+    }
+#  endif
 #endif	/* ENCRYPTION */
+    /* Options like environment require authentication
+     * and encryption to be completed 
+     */
+    auth_negotiated = 1;
+
+#ifdef  FWD_X
+    /* TODO: don't send this if not ENCRYPT or TLS */
+    if (!fwdx_disable_flag) {
+	if (!fwdx_sbdata)
+	    fwdx_sbdata = malloc(fwdx_sbdata_size);
+	if (fwdx_sbdata
+#ifdef ENCRYPTION
+             && (
+#ifdef TLS
+                 tls_active || 
+#endif /* TLS */
+                 encrypt_is_encrypting())
+#endif /* ENCRYPTION */
+             )
+	    send_will(TELOPT_FORWARD_X, 1);
+    }
+#endif  /* FWD_X */
     send_do(TELOPT_TTYPE, 1);
     send_do(TELOPT_TSPEED, 1);
     send_do(TELOPT_XDISPLOC, 1);
     send_do(TELOPT_NEW_ENVIRON, 1);
     send_do(TELOPT_OLD_ENVIRON, 1);
-    while (
-#ifdef	ENCRYPTION
-	   his_do_dont_is_changing(TELOPT_ENCRYPT) ||
-           his_will_wont_is_changing(TELOPT_ENCRYPT) ||
-#endif	/* ENCRYPTION */
-	   his_will_wont_is_changing(TELOPT_TTYPE) ||
+    while (his_will_wont_is_changing(TELOPT_TTYPE) ||
 	   his_will_wont_is_changing(TELOPT_TSPEED) ||
 	   his_will_wont_is_changing(TELOPT_XDISPLOC) ||
+#ifdef  FWD_X
+	   /* if fwdx_sbdata == NULL, we didn't send WILL FORWARD_X */
+	   (fwdx_sbdata && his_do_dont_is_changing(TELOPT_FORWARD_X)) ||
+#endif
 	   his_will_wont_is_changing(TELOPT_NEW_ENVIRON) ||
 	   his_will_wont_is_changing(TELOPT_OLD_ENVIRON)) {
 	ttloop();
     }
-#ifdef	ENCRYPTION
-    /*
-     * Wait for the negotiation of what type of encryption we can
-     * send with.  If autoencrypt is not set, this will just return.
-     */
-    if (his_state_is_will(TELOPT_ENCRYPT)) {
-	encrypt_wait();
-    }
-    if (auth_must_encrypt()) {
-	time_t timeout = time(0) + 60;
 	
-	if (my_state_is_dont(TELOPT_ENCRYPT) ||
-	    my_state_is_wont(TELOPT_ENCRYPT))
-	    encrypt_failure();
-
-	if (!EncryptStartInput() || !EncryptStartOutput())
-	    encrypt_failure();
-
-	while (!encrypt_is_encrypting()) {
-	    if (time(0) > timeout)
-		encrypt_failure();
-	    ttloop();
-	}
-    }
-#endif	/* ENCRYPTION */
-    /* Options like environment require authentication and encryption
-    negotiation to be completed.*/
-    auth_negotiated = 1;
-
     if (his_state_is_will(TELOPT_TSPEED)) {
 	static unsigned char sb[] =
 			{ IAC, SB, TELOPT_TSPEED, TELQUAL_SEND, IAC, SE };
@@ -832,7 +1051,7 @@ getterminaltype(name)
 		/*
 		 * Save the unknown name, and request the next name.
 		 */
-		(void) strncpy(last, terminaltype, sizeof(last));
+		(void) strncpy(last, terminaltype, sizeof(last) - 1);
                 last[sizeof(last) - 1] = '\0';
 		_gettermname();
 		if (terminaltypeok(terminaltype))
@@ -851,9 +1070,10 @@ getterminaltype(name)
 		     * the start of the list.
 		     */
 		     _gettermname();
-		    if (strncmp(first, terminaltype, sizeof(first)) != 0)
-			(void) strncpy(terminaltype, first, sizeof(first));
+		    if (strncmp(first, terminaltype, sizeof(first)) != 0) {
+			(void) strncpy(terminaltype, first, sizeof(terminaltype) - 1);
                     terminaltype[sizeof(terminaltype) - 1] = '\0';
+                    }
 		    break;
 		}
 	    }
@@ -885,7 +1105,11 @@ _gettermname()
 terminaltypeok(s)
     char *s;
 {
-    char buf[1024];
+#ifdef TERMCAP
+    char buf[4096];
+#else
+    int err;
+#endif /* TERMCAP */
 
     if (!s || !*s)
 	return(1);
@@ -897,8 +1121,13 @@ terminaltypeok(s)
      * it won't help to say we failed, because we won't be
      * able to verify anything else.  So, we treat -1 like 1.
      */
+#ifdef TERMCAP
     if (tgetent(buf, s) == 0)
 	return(0);
+#else
+    if(setupterm(s, 1, &err) < 0)
+      return 0;
+#endif /* TERMCAP */
     return(1);
 }
 
@@ -940,6 +1169,9 @@ doit(who)
 	int level;
 	int ptynum;
 	char user_name[256];
+#ifdef TLS
+	char *user = NULL;
+#endif /* TLS */
 
 	/*
 	 * Find an available pty to use.
@@ -956,8 +1188,9 @@ doit(who)
 		if ((lp = getpty()) == NULL)
 			fatal(net, "Out of ptys");
 
-		if ((pty = open(lp, 2)) >= 0) {
-			strcpy(line,lp);
+		if ((pty = open(lp, O_RDWR)) >= 0) {
+			strncpy(line, lp, sizeof(line) - 1);
+			line[sizeof(line) - 1] = 0;
 			line[5] = 't';
 			break;
 		}
@@ -1013,6 +1246,51 @@ doit(who)
 
 #if	defined(AUTHENTICATION) || defined(ENCRYPTION)
 	auth_encrypt_init(hostname, host, "TELNETD", 1);
+#endif
+#ifdef TLS
+	if (!tls_active) {
+		send_do(TELOPT_START_TLS, 1);
+		while (his_will_wont_is_changing(TELOPT_START_TLS))
+			ttloop();
+		while (his_state_is_will(TELOPT_START_TLS) && !tls_follows_from_client )
+			ttloop();
+		if (tls_follows_from_client) {
+			static unsigned char sb[] = { IAC, SB, TELOPT_START_TLS, TLS_FOLLOWS, IAC, SE };
+			memmove(nfrontp, sb, sizeof sb);
+			nfrontp += sizeof sb;
+			DIAG(TD_OPTIONS, printsub('>', sb + 2, sizeof sb - 2););
+			netflush();
+			if (!tls_start()) {
+				/* reset any negotiated telnet options */
+				memset(options, 0, sizeof(options));
+				init_env();
+#if	defined(AUTHENTICATION) || defined(ENCRYPTION)
+				auth_encrypt_init(hostname, host, "TELNETD", 1);
+#endif
+			}
+			else { /* quit if error at TLS neg. */
+			    sleep(1);
+			    clean_exit(1);
+			}
+		}
+		if (!tls_active && tls_required)
+			clean_exit(1);
+	}
+#  if	defined(AUTHENTICATION) || defined(ENCRYPTION)
+	/*
+	 * Combining TLS and AUTHENTICATION creates a weird chicken-and-egg
+	 * scenario - if TLS client auth is being attempted, we need to
+	 * get the username, which requires us to negotiate the ENVIRON
+	 * options, but once we do that, we aren't supposed to
+	 * renegotiate AUTH.  The problem is that we don't know yet
+	 * if the cert will be accepted for login before we burn our
+	 * bridges and negotiate for the username.  If TLS client auth
+	 * is attempted and fails, we will not attempt to fall back to
+	 * Telnet AUTHENTICATION.
+	 */
+	if(tls_active && tls_using_client_auth())
+	  auth_ignore = 1;
+#  endif
 #endif
 
 	init_env();
@@ -1034,6 +1312,28 @@ doit(who)
 	 * Start up the login process on the slave side of the terminal
 	 */
 #ifndef	convex
+#ifdef TLS
+	user = getenv("USER");
+	if (tls_userid_from_client_cert()) {
+		if (!user || !strcmp(tls_userid_from_client_cert(), user)) {
+			user = tls_userid_from_client_cert();
+			setenv("USER", user, 0);
+			syslog(LOG_NOTICE,"User %s logged in by TLS/X509 authentication", user);
+			startslave(host, AUTH_VALID, user);
+		}
+		else if (tls_is_user_valid(user)) {
+			syslog(LOG_NOTICE,"User %s logged in by TLS/X509 authentication", user);
+			startslave(host, AUTH_VALID, user);
+		}
+		else
+			startslave(host, level, user_name);
+	}
+	else if (user && tls_is_user_valid(user)) {
+		syslog(LOG_NOTICE,"User %s logged in by TLS/X509 authentication", user);
+		startslave(host, AUTH_VALID, user);
+	}
+	else
+#endif
 	startslave(host, level, user_name);
 
 #if	defined(_SC_CRAY_SECURE_SYS)
@@ -1083,10 +1383,14 @@ telnet(f, p, host)
 #endif
 {
 	int on = 1;
+#ifdef TLS
+	int pnd;
+#endif /* TLS */
 #define	TABBUFSIZ	512
 	char	defent[TABBUFSIZ];
 	char	defstrs[TABBUFSIZ];
 #undef	TABBUFSIZ
+#undef HE /* Make sure its not defined as a macro. */
 	char *HE;
 	char *HN;
 	char *IM;
@@ -1179,7 +1483,7 @@ telnet(f, p, host)
 	 */
 	if (his_want_state_is_will(TELOPT_ECHO)) {
 		DIAG(TD_OPTIONS,
-			{sprintf(nfrontp, "td: simulating recv\r\n");
+			{net_printf("td: simulating recv\r\n");
 			 nfrontp += strlen(nfrontp);});
 		willoption(TELOPT_ECHO);
 	}
@@ -1239,6 +1543,7 @@ telnet(f, p, host)
 #endif
 
 	(void) signal(SIGCHLD, cleanup);
+	(void) signal(SIGPIPE, cleanup);
 
 #if	defined(CRAY2) && defined(UNICOS5)
 	/*
@@ -1292,7 +1597,8 @@ telnet(f, p, host)
 		HN = getstr("hn", &cp);
 		IM = getstr("im", &cp);
 		if (HN && *HN)
-			(void) strcpy(host_name, HN);
+			strncpy(host_name, HN, sizeof(host_name));
+		host_name[sizeof(host_name) - 1] = 0;
 		if (IM == 0)
 			IM = "";
 	} else if((ifd = open("/etc/issue.net", O_RDONLY)) > 0 ||
@@ -1308,7 +1614,31 @@ telnet(f, p, host)
 		close(ifd);
 		HE = 0;
 	} else {
+#ifdef HAVE_UNAME
+		struct utsname u;
+#endif
+
+#ifdef DEFAULT_IM
 		IM = DEFAULT_IM;
+#else
+		IM = 0;
+#ifdef HAVE_UNAME
+		if (uname (&u) == 0) {
+			IM = malloc (strlen (UNAME_IM_PREFIX)
+				     + strlen (u.sysname)
+				     + 1 + strlen (u.release)
+				     + strlen (UNAME_IM_SUFFIX) + 1);
+			if (IM)
+				sprintf (IM, "%s%s %s%s",	/* safe */
+					 UNAME_IM_PREFIX,
+					 u.sysname, u.release,
+					 UNAME_IM_SUFFIX);
+		}
+#endif /* HAVE_UNAME */
+		if (! IM)
+			IM = "\r\n\nUNIX (%h) (%t)\r\n\n";
+#endif /* DEFAULT_IM */
+
 		HE = 0;
 	}
 	edithost(HE, host_name);
@@ -1328,14 +1658,14 @@ telnet(f, p, host)
 #endif	/* LINEMODE */
 
 	DIAG(TD_REPORT,
-		{sprintf(nfrontp, "td: Entering processing loop\r\n");
+		{net_printf("td: Entering processing loop\r\n");
 		 nfrontp += strlen(nfrontp);});
 
 #ifdef	convex
 	startslave(host);
 #endif
 
-	nfd = ((f > p) ? f : p) + 1;
+	nfd = ((f > p) ? f : p);
 	for (;;) {
 		fd_set ibits, obits, xbits;
 		register int c;
@@ -1358,6 +1688,10 @@ telnet(f, p, host)
 			FD_SET(f, &obits);
 		} else {
 			FD_SET(p, &ibits);
+#ifdef FWD_X
+			fwdx_init_fd_set(&ibits, &obits);
+			nfd = fwdx_max_socket(nfd);
+#endif
 		}
 		if (pfrontp - pbackp || ncc > 0) {
 			FD_SET(p, &obits);
@@ -1367,7 +1701,7 @@ telnet(f, p, host)
 		if (!SYNCHing) {
 			FD_SET(f, &xbits);
 		}
-		if ((c = select(nfd, &ibits, &obits, &xbits,
+		if ((c = select(nfd + 1, &ibits, &obits, &xbits,
 						(struct timeval *)0)) < 1) {
 			if (c == -1) {
 				if (errno == EINTR) {
@@ -1429,41 +1763,62 @@ telnet(f, p, host)
 
 			(void) ioctl(net, SIOCATMARK, (char *)&atmark);
 			if (atmark) {
-			    ncc = recv(net, netibuf, sizeof (netibuf), MSG_OOB);
+			    ncc = RECV(net, netibuf, sizeof (netibuf), MSG_OOB);
 			    if ((ncc == -1) && (errno == EINVAL)) {
-				ncc = read(net, netibuf, sizeof (netibuf));
+				ncc = READ(net, netibuf, sizeof (netibuf));
 				if (sequenceIs(didnetreceive, gotDM)) {
 				    SYNCHing = stilloob(net);
 				}
 			    }
 			} else {
-			    ncc = read(net, netibuf, sizeof (netibuf));
+			    ncc = READ(net, netibuf, sizeof (netibuf));
 			}
 		    } else {
-			ncc = read(net, netibuf, sizeof (netibuf));
+			ncc = READ(net, netibuf, sizeof (netibuf));
 		    }
 		    settimer(didnetreceive);
 #else	/* !defined(SO_OOBINLINE)) */
-		    ncc = read(net, netibuf, sizeof (netibuf));
+		    ncc = READ(net, netibuf, sizeof (netibuf));
 #endif	/* !defined(SO_OOBINLINE)) */
-		    if (ncc < 0 && errno == EWOULDBLOCK)
+		    if (ncc < 0 && (errno == EWOULDBLOCK || errno == EINTR))
 			ncc = 0;
 		    else {
-			if (ncc <= 0) {
+			if (ncc <= 0)
 			    break;
-			}
 			netip = netibuf;
 		    }
 		    DIAG((TD_REPORT | TD_NETDATA),
-			    {sprintf(nfrontp, "td: netread %d chars\r\n", ncc);
+			    {net_printf("td: netread %d chars\r\n", ncc);
 			     nfrontp += strlen(nfrontp);});
 		    DIAG(TD_NETDATA, printdata("nd", netip, ncc));
 		}
 
+#ifdef FWD_X
+		/*
+		 * Any "X-server" connections / data waiting?
+		 */
+		 fwdx_check_sockets(&ibits, &obits);
+#endif
+
 		/*
 		 * Something to read from the pty...
 		 */
 		if (FD_ISSET(p, &ibits)) {
+		/* to find out when the remote user logs in (so we can change
+		 * ownership of the temporary xauth file), we monitor when the
+		 * ownership of the pty changes.
+		 */
+			static int stat_count = 0;
+			if (login_uid < 0 && probe_uid_needed() && stat_count < 2000) {
+				struct stat sbuf;
+				stat(line, &sbuf);
+				if (sbuf.st_uid) { /* not root (0) anymore */
+					login_uid = sbuf.st_uid;
+					login_gid = sbuf.st_gid;
+					do_uid_callbacks(login_uid, login_gid);
+				}
+				stat_count++;  /* we only try a maximun number of times */
+			}
 #ifndef	STREAMSPTY
 			pcc = read(p, ptyibuf, BUFSIZ);
 #else
@@ -1484,7 +1839,7 @@ telnet(f, p, host)
 				if (pcc <= 0)
 					break;
 #if	!defined(CRAY2) || !defined(UNICOS5)
-#ifdef	LINEMODE
+#if defined(LINEMODE) && defined(TIOCPKT_IOCTL)
 				/*
 				 * If ioctl from pty, pass it through net
 				 */
@@ -1493,7 +1848,7 @@ telnet(f, p, host)
 					localstat();
 					pcc = 1;
 				}
-#endif	/* LINEMODE */
+#endif	/* LINEMODE && TIOCPKT_IOCTL */
 				if (ptyibuf[0] & TIOCPKT_FLUSHWRITE) {
 					netclear();	/* clear buffer back */
 #ifndef	NO_URGENT
@@ -1518,8 +1873,7 @@ telnet(f, p, host)
 					    ptyibuf[0] & TIOCPKT_DOSTOP ? 1 : 0;
 					if (newflow != flowmode) {
 						flowmode = newflow;
-						(void) sprintf(nfrontp,
-							"%c%c%c%c%c%c",
+						(void) net_printf("%c%c%c%c%c%c",
 							IAC, SB, TELOPT_LFLOW,
 							flowmode ? LFLOW_ON
 								 : LFLOW_OFF,
@@ -1546,7 +1900,7 @@ telnet(f, p, host)
 		}
 
 		while (pcc > 0) {
-			if ((&netobuf[BUFSIZ] - nfrontp) < 2)
+			if ((&netobuf[netobuf_size] - nfrontp) < 2)
 				break;
 			c = *ptyip++ & 0377, pcc--;
 			if (c == IAC)
@@ -1583,6 +1937,53 @@ telnet(f, p, host)
 			telrcv();
 		if (FD_ISSET(p, &obits) && (pfrontp - pbackp) > 0)
 			ptyflush();
+
+#ifdef TLS
+                while ( ((pnd = tls_pending()) > 0) && (nfrontp - nbackp == 0) &&
+			(pfrontp - pbackp == 0) ) {
+                    DIAG((TD_REPORT | TD_NETDATA),
+                          {net_printf("td: tls_pending %d chars\r\n", pnd);
+                              nfrontp += strlen(nfrontp);});
+                    
+		    ncc = READ(net, netibuf, sizeof(netibuf));
+                    if (ncc <= 0)
+                        break;
+                    netip = netibuf;
+                    
+		    DIAG((TD_REPORT | TD_NETDATA),
+                          {net_printf("td: tls_pending netread %d chars\r\n", ncc);
+                              nfrontp += strlen(nfrontp);});
+                    DIAG(TD_NETDATA, printdata("nd", netip, ncc));
+                    if (ncc > 0)
+                        telrcv();
+
+                    FD_ZERO(&ibits);
+                    FD_ZERO(&obits);
+                    /*
+                    * Never look for input if there's still
+                    * stuff in the corresponding output buffer
+                    */
+                    FD_SET(f, &obits);
+                    FD_SET(p, &obits);
+                    if ((c = select(((f > p) ? f : p) + 1, &ibits, &obits, NULL,
+                                     (struct timeval *)0)) < 1) {
+			if (c == -1) {
+                            if (errno == EINTR) {
+                                continue;
+                            }
+			}
+			sleep(5);
+			continue;
+                    }
+
+                    if (FD_ISSET(f, &obits) && (nfrontp - nbackp) > 0)
+			netflush();
+                    if (ncc > 0)
+                        telrcv();
+                    if (FD_ISSET(p, &obits) && (pfrontp - pbackp) > 0)
+                        ptyflush();
+                }
+#endif /* TLS */
 	}
 	cleanup(0);
 }  /* end of telnet */
@@ -1756,7 +2157,8 @@ recv_ayt()
 		return;
 	}
 #endif
-	(void) strcpy(nfrontp, "\r\n[Yes]\r\n");
+	(void) strncpy(nfrontp, "\r\n[Yes]\r\n",
+                        sizeof(netobuf) - 1 - (nfrontp - netobuf));
 	nfrontp += 9;
 }
 
